//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License
//

#pragma once

namespace VectorWorks
{
	namespace Extension
	{
		// ------------------------------------------------------------------------
		enum ETINDisplay {
			kTINDisplayNothing,
			kTINDisplayExistingOnly,
			kTINDisplayProposedOnly,
			kTINDisplayExistingAndProposed,
			kTINDisplayCutAndFill
		};

		// ------------------------------------------------------------------------
		enum E2DTINStyle {
			k2DTINStyle2DContour,
			k2DTINStyle2DContourSmoothed,
			k2DTINStyle2DContourColoredElevations,
			k2DTINStyle2DTriangles,
			k2DTINStyle2DTrianglesColoredSlopes,
			k2DTINStyle2DTriangulatedContours,
			k2DTINStyle2DTriangulatedContoursColoredSlopes
		};

		// ------------------------------------------------------------------------
		enum E3DTINStyle {
			k3DTINStyleContour,
			k3DTINStyleGrid,
			k3DTINStyleMeshSolid,
			k3DTINStyleTriangles,
			k3DTINStyleTrianglesColoredSlopes,
			k3DTINStyleExtrudedContour,
			k3DTINStyleTriangulatedContours,
			k3DTINStyleTriangulatedContoursColoredSlopes
		};

		// ------------------------------------------------------------------------
		enum EModifiersSource {
			kModifiersSourceAllLayers,
			kModifiersSourceVisibleLayers,
			kModifiersSourceObjectLayer
		};

		// ------------------------------------------------------------------------
		struct SDTMCommColor
		{
			UInt8		red;
			UInt8		green;
			UInt8		blue;
		};

		// ------------------------------------------------------------------------
		struct SDTMLineAttributes
		{
			UInt16		lineStyle;
			UInt16		lineWeight;
		};

		// ---------------------------------------------------------------
		struct SDTMMarkerAttributes {
			MarkerType		style;
			Sint8			nAngle;
			double			dSize;
			double			dWidth;
			Sint8			nThicknessBasis;
			double			dThickness;
		};

		// ------------------------------------------------------------------------
		struct SDTMContourData
		{
			UInt32					fnSize;

			double					fdContourInterval;	// contour interval
			UInt16					fnMajMultiple;		// major multiple
			double					fdDatumElevation;	// datum elevation
			double					fdBaseElevation;	// base elevation
			double					fdMinElevation;		// minimum elevation
			double					fdMaxElevation;		// maximum elevation

			double					fdSmoothInterval;	// smoothing of contours
		};

		// ------------------------------------------------------------------------
		struct SDTMVisualData
		{
			UInt32					fnSize;

			ETINDisplay				fnDisplay2D;
			E2DTINStyle				fnStyle2D;
			bool					fbShowFlowArrows;
			double					fdFlowArrowsSpacing;
			bool					fbShowLabes;
			bool					fbShowLabesAlongLine;
			bool					fbDrawHull;
			ETINDisplay				fnDisplay3D;
			E3DTINStyle				fnStyle3D;
			bool					fbShow3DGrid;
			double					fdGridSpacing;
			bool					fbGridFill;
			bool					fbShow3DContour;
		};

		// ------------------------------------------------------------------------
		struct SDTMGraphicData
		{
			UInt32					fnSize;

			TXString				f2DContoursMajorExstClass;
			TXString				f2DContoursMajorPropClass;
			SDTMCommColor			f2DContoursMajorColor;
			SDTMLineAttributes		f2DContoursMajorExstAttr;
			SDTMLineAttributes		f2DContoursMajorPropAttr;

			TXString				f2DContoursMinorExstClass;
			TXString				f2DContoursMinorPropClass;
			SDTMCommColor			f2DContoursMinorColor;
			SDTMLineAttributes		f2DContoursMinorExstAttr;
			SDTMLineAttributes		f2DContoursMinorPropAttr;

			TXString				f2DFlowArrowsClass;
			SDTMCommColor			f2DFlowArrowsColor;
			SDTMLineAttributes		f2DFlowArrowsAttr;
			SDTMMarkerAttributes	f2DFlowArrowsMarker;

			TXString				f2DHullClass;
			SDTMCommColor			f2DHullColor;
			SDTMLineAttributes		f2DHullAttr;

			TXString				f2DTrianglesClass;
			SDTMCommColor			f2DTrianglesColor;
			SDTMLineAttributes		f2DTrianglesAttr;

			TXString				f2DContourLabelsClass;
			SDTMCommColor			f2DContourLabelsColor;

			TXString				f3DContoursMajorClass;
			SDTMCommColor			f3DContoursMajorColor;
			SDTMLineAttributes		f3DContoursMajorAttr;

			TXString				f3DContoursMinorClass;
			SDTMCommColor			f3DContoursMinorColor;
			SDTMLineAttributes		f3DContoursMinorAttr;

			TXString				f3DTrianglesClass;
			SDTMCommColor			f3DTrianglesColor;
			SDTMLineAttributes		f3DTrianglesAttr;

			TXString				f3DGridClass;
			SDTMCommColor			f3DGridColor;
			SDTMLineAttributes		f3DGridAttr;

			TXString				f3DFillClass;
			SDTMCommColor			f3DFillColor;

			TXString				f3DSidesBottomClass;
			SDTMCommColor			f3DSidesBottomColor;

			TXString				f3DCutVolumeClass;
			SDTMCommColor			f3DCutVolumeColor;

			TXString				f3DFillVolumeClass;
			SDTMCommColor			f3DFillVolumeColor;
		};

		// ------------------------------------------------------------------------
		struct SDTMSiteSlopeAnalysis
		{
			UInt32					fnSize;

			UInt8					fnNumSlopeCategories;
			double					farrSlopes[ 10 ];
			SDTMCommColor			farrSlopeColors[ 10 ];

			SDTMCommColor			fExceedingSlopeColor;
			SDTMCommColor			fMinElevationColor;
			SDTMCommColor			fMaxElevationColor;
		};

		// ------------------------------------------------------------------------
		struct SDTMTexturing
		{
			UInt32					fnSize;

			// texturing
			UInt32					fSurfaceTextureRef;
		};

		// ---------------------------------------------------------------------------
		// {361A3F29-E180-4dde-9544-D84C96086B1A}
		static const VWIID IID_DTMSettings = { 0x361a3f29, 0xe180, 0x4dde, { 0x95, 0x44, 0xd8, 0x4c, 0x96, 0x8, 0x6b, 0x1a } };

		class DYNAMIC_ATTRIBUTE IDTMSettings : public IVWUnknown
		{
		public:
			virtual VCOMError VCOM_CALLTYPE GetContourData(SDTMContourData& outData) = 0;
			virtual VCOMError VCOM_CALLTYPE SetContourData(const SDTMContourData& data) = 0;
			virtual VCOMError VCOM_CALLTYPE GetVisualData(SDTMVisualData& outData) = 0;
			virtual VCOMError VCOM_CALLTYPE SetVisualData(const SDTMVisualData& data) = 0;
			virtual VCOMError VCOM_CALLTYPE GetGraphicData(SDTMGraphicData& outData) = 0;
			virtual VCOMError VCOM_CALLTYPE SetGraphicData(const SDTMGraphicData& data) = 0;
			virtual VCOMError VCOM_CALLTYPE GetSlopeAnalysisData(SDTMSiteSlopeAnalysis& outData) = 0;
			virtual VCOMError VCOM_CALLTYPE SetSlopeAnalysisData(const SDTMSiteSlopeAnalysis& data) = 0;
			virtual VCOMError VCOM_CALLTYPE GetTexturingData(SDTMTexturing& outData) = 0;
			virtual VCOMError VCOM_CALLTYPE SetTexturingData(const SDTMTexturing& data) = 0;
		};
	}
}