//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License


#pragma once


namespace VectorWorks
{
	namespace Extension
	{
		// ----------------------------------------------------------------------------------------------------
		typedef VWIID	TSinkIID;

		// ----------------------------------------------------------------------------------------------------
		// Base class for extension sink
		class DYNAMIC_ATTRIBUTE IEventSink : public IVWUnknown
		{
		public:
		};

		// ----------------------------------------------------------------------------------------------------
		// Base class for Vectorworks extensions implementation
		class DYNAMIC_ATTRIBUTE IExtension : public IVWUnknown
		{
		public:
			virtual const TXString& VCOM_CALLTYPE	GetUniversalName() = 0;
			virtual Sint32 VCOM_CALLTYPE			GetVersion() = 0;
			virtual IEventSink* VCOM_CALLTYPE		QueryEventSink(const TSinkIID& iid) = 0;
		};

		// ----------------------------------------------------------------------------------------------------
		template<class T> class VCOMSinkPtr : public VCOMPtr< T >
		{
		public:
			VCOMSinkPtr()					: VCOMPtr< T >()					{ }
			VCOMSinkPtr(T* ptr)				: VCOMPtr< T >( ptr )				{ }
			VCOMSinkPtr(const VWIID& iid)	: VCOMPtr< T >( iid )				{ }
			VCOMSinkPtr(IExtension* extension, const TSinkIID& iid)				{ this->Query( extension, iid ); }
			VCOMSinkPtr(const VCOMSinkPtr& p)									{ T* ptr = p.Get(); this->Set( ptr ); if ( ptr ) { ptr->AddRef(); } }

		public:
			VCOMError	Query(IExtension* extension, const TSinkIID& iid)	{
															this->Release();

															VCOMError err = kVCOMError_Failed;
															if ( extension == NULL ) {
																err = kVCOMError_InvalidArg;
															}
															else {
																IEventSink*	pInterface	= extension->QueryEventSink( iid );
																// Vlado: Temporally removed 'dynamic_cast'
																// because on new Mac OS it is failing to cast properly
																//fPtr = dynamic_cast<T*>( pInterface );
																this->Set( (T*)( pInterface ) );
																if ( this->Get() == NULL ) {
																	if ( pInterface ) {
																		pInterface->Release();
																		pInterface	= NULL;
																	}
																	// the passed 'iid' is not from the extected type 'T'
																	err			= kVCOMError_InvalidArg;
																}
                                                                else
                                                                    err = kVCOMError_NoError;
															}
															return err;
														}
		};

		typedef VCOMSinkPtr<IEventSink>					IEventSinkPtr;

		// ----------------------------------------------------------------------------------------------------
		// {23D80C42-DC4D-4d1d-BFF2-CAE178A54059}
		static const TSinkIID IID_LegacyEventSink = { 0x23d80c42, 0xdc4d, 0x4d1d, { 0xbf, 0xf2, 0xca, 0xe1, 0x78, 0xa5, 0x40, 0x59 } };
		// ----------------------------------------------------------------------------------------------------
		class DYNAMIC_ATTRIBUTE ILegacyEventSink : public IEventSink
		{
		public:
			virtual long VCOM_CALLTYPE				LegacyMain(long action, long message1, long message2) = 0;
		};

	}
}
