//
//	IIFCSupport.h
//
//	Copyright  Nemetschek North America, Inc  2005.
//	All Rights Reserved.

#pragma once



// fake out the VCOM for VW12.5.x
#ifdef _VWFC_FOR_VW125x


#	ifndef VCOM_CALLTYPE
#		define		VCOM_CALLTYPE
#	endif

#	define VCOMPtr		FakeVCOMPtr

	template<class T> class FakeVCOMPtr
	{
	public:
						FakeVCOMPtr()						{ fPtr = NULL; }
						FakeVCOMPtr(T* p)					{ fPtr = p; if ( fPtr ) { fPtr->AddRef(); } }
						FakeVCOMPtr(const VWIID& iid)		{ fPtr = NULL; this->Query( iid ); }
						FakeVCOMPtr(const FakeVCOMPtr& p)	{ fPtr = p.fPtr; if ( fPtr ) { fPtr->AddRef(); } }
						~FakeVCOMPtr()						{ if ( fPtr ) { fPtr->Release(); } this->IFC_ReleaseInterfaces(); }

		FakeVCOMPtr&	operator=(T* ptr)					{ if ( fPtr ) { fPtr->Release(); } fPtr = ptr; if ( fPtr ) { fPtr->AddRef(); } return *this; }
		FakeVCOMPtr&	operator=(const FakeVCOMPtr& p)		{ if ( fPtr ) { fPtr->Release(); } fPtr = p.fPtr; if ( fPtr ) { fPtr->AddRef(); } return *this; }

		T*				operator->() const					{ return fPtr; }
		T**				operator&()							{ return & fPtr; }
						operator T*() const 				{ return fPtr; }

	private:
		void		Query(const VWIID& iid)
		{
			PluginLibraryArgTable	callTable;
			callTable.args[0].argType		= kVoidPtr;
			callTable.args[0].voidData		= (void*) gCBP;
			callTable.args[1].argType		= kVoidPtr;
			callTable.args[1].voidData		= (void*) & iid;
			if ( ::GS_CallPluginLibrary( gCBP, TXString("IFC_QueryInterface"), (PluginLibraryArgTable*) & callTable, 0 ) ) {
				fPtr	= (T*) callTable.functionResult.voidData;
				if ( fPtr ) {
					fPtr->AddRef();
				}
			}
		}

		void		IFC_ReleaseInterfaces()
		{
			if ( fPtr && fPtr->Release() == 0 ) {
				PluginLibraryArgTable	callTable;
				callTable.args[0].argType		= kVoidPtr;
				callTable.args[0].voidData		= (void*) gCBP;
				callTable.args[1].argType		= kVoidPtr;
				callTable.args[1].voidData		= (void*) fPtr;
				VERIFYN( kVStanev, ::GS_CallPluginLibrary( gCBP, TXString("IFC_ReleaseInterfaces"), (PluginLibraryArgTable*) & callTable, 0 ) );
			}
		}
	private:
		T*		fPtr;
	};



	// ------------------------------------------------------------------------------------------------------------------------
	static const VWIID IID_ProgressDialog = { 0x7E8C933D, 0xCC09, 0x4202, { 0xB1, 0x4F, 0xEF, 0xFB, 0x04, 0xFA, 0x42, 0x20 } };
	class DYNAMIC_ATTRIBUTE IProgressDialog : public IVWUnknown
	{
	public:
		virtual VCOMError VCOM_CALLTYPE	Open(const TXString& title)						= 0;
		virtual VCOMError VCOM_CALLTYPE Close()											= 0;

		virtual VCOMError VCOM_CALLTYPE	AllowUserCancel(bool bAllow)					= 0;
		virtual VCOMError VCOM_CALLTYPE	ResetMeter(Uint16 meter)						= 0;
		virtual VCOMError VCOM_CALLTYPE	SetTopText(const TXString& szTopText)			= 0;
		virtual VCOMError VCOM_CALLTYPE	SetBottomText(const TXString& szBottomText)		= 0;
		virtual VCOMError VCOM_CALLTYPE	SetMeterText(const TXString& szMeterText)		= 0;

		virtual VCOMError VCOM_CALLTYPE	HasUserCanceled(bool& bUserCanceled)			= 0;
		virtual VCOMError VCOM_CALLTYPE	DoYield()										= 0;
		virtual VCOMError VCOM_CALLTYPE	IncrementMeter(Uint16 step)						= 0;
		
	};

	// ------------------------------------------------------------------------------------------------------------------------
	static const VWIID IID_FileIdentifier = { 0xB4B088B1, 0x3793, 0x11DB, { 0x9C, 0x35, 0x00, 0x50, 0x8D, 0x5E, 0x98, 0x51 } };
	class DYNAMIC_ATTRIBUTE IFileIdentifier : public IVWUnknown
	{
	public:
		virtual VCOMError VCOM_CALLTYPE	SetFileFullPath(const TXString& path) = 0;
		virtual VCOMError VCOM_CALLTYPE	GetFileFullPath(TXString& outPath) = 0;
		virtual VCOMError VCOM_CALLTYPE	GetFileExtension(TXString& outExtension) = 0;
	};

	// ------------------------------------------------------------------------------------------------------------------------
	static const VWIID IID_RawOSFile = { 0x4F7F77DD, 0x3796, 0x11DB, { 0x9C, 0x35, 0x00, 0x50, 0x8D, 0x5E, 0x98, 0x51 } };
	class DYNAMIC_ATTRIBUTE IRawOSFile : public IVWUnknown
	{
	public:
		virtual VCOMError VCOM_CALLTYPE	Open(IFileIdentifier* pFileID, bool bReadable, bool bWritable, bool bRandomAccess, bool bTruncateExisting) = 0;
		virtual VCOMError VCOM_CALLTYPE	Close() = 0;
		virtual VCOMError VCOM_CALLTYPE	GetFileSize(Uint64& outValue) = 0;
		virtual VCOMError VCOM_CALLTYPE	Read(Uint64 position, Uint64& inoutSize, void* pOutBuffer) = 0;
		virtual VCOMError VCOM_CALLTYPE	Write(Uint64 position, Uint64 size, const void* pBuffer) = 0;
	};

	// ------------------------------------------------------------------------------------------------------------------------
	static const VWIID IID_FileChooserDialog = { 0x885F311B, 0x3795, 0x11DB, { 0x9C, 0x35, 0x00, 0x50, 0x8D, 0x5E, 0x98, 0x51 } };
	class DYNAMIC_ATTRIBUTE IFileChooserDialog : public IVWUnknown
	{
	public:
		virtual VCOMError VCOM_CALLTYPE	SetTitle(const TXString& title) = 0;
		virtual VCOMError VCOM_CALLTYPE	SetDefaultFileName(const TXString& name) = 0;
		virtual VCOMError VCOM_CALLTYPE	GetSelectedFileName(Uint32 fileIndex, IFileIdentifier** ppOutFileID) = 0;
		virtual VCOMError VCOM_CALLTYPE	AddFilterAllFiles() = 0;
		virtual VCOMError VCOM_CALLTYPE	AddFilter(const TXString& filterExt, const TXString& filterDesc) = 0;

		virtual VCOMError VCOM_CALLTYPE	RunOpenDialog() = 0;
		virtual VCOMError VCOM_CALLTYPE	RunSaveDialog() = 0;
	};
#endif



namespace VectorWorks
{
	namespace IFCLib
	{


		// ----------------------------------------------------------------------------------------
		enum EIFCSupportSchema {
			kIFCSupportScheme_2x2,
			kIFCSupportScheme_2x3
		};

		// ---------------------------------------------------------------------------
		// {E2B50992-BE66-46a8-AA5E-48756374B462}
		static const VWIID IID_IFCSupport = { 0xe2b50992, 0xbe66, 0x46a8, { 0xaa, 0x5e, 0x48, 0x75, 0x63, 0x74, 0xb4, 0x62 } };

		class DYNAMIC_ATTRIBUTE IIFCSupport : public IVWSingletonUnknown
		{
		public:
			virtual VCOMError VCOM_CALLTYPE SetIFCScheme(EIFCSupportSchema scheme) = 0;
			virtual VCOMError VCOM_CALLTYPE GetIFCObjectData(MCObjectHandle hObject, TXString& outIfcObjName, TXString& outIfcObjectUUID) = 0;
			virtual VCOMError VCOM_CALLTYPE UpdateVWDocumentIFCData() = 0;
			virtual VCOMError VCOM_CALLTYPE	CloseForDocument(MCObjectHandle hDocument) = 0;
			virtual VCOMError VCOM_CALLTYPE ShowObjectIFCDataEditDlg(MCObjectHandle hObject, const TXString& strIfcObjName, bool bCreateData, bool bAllowTypeChange=true) = 0;
			virtual VCOMError VCOM_CALLTYPE ShowPickIFCTypeDlg(TXString& outIfcObjName) = 0;

			virtual VCOMError VCOM_CALLTYPE ShowExportOptions(bool bExportSingleObjects) = 0;

			virtual VCOMError VCOM_CALLTYPE ImportFile(IFileIdentifier* pFileID, bool bShowUI) = 0;
			virtual VCOMError VCOM_CALLTYPE ExportFile(IFileIdentifier* pFileID, bool bShowUI) = 0;
			

			// VS/SDK support
			virtual VCOMError VCOM_CALLTYPE IFC_SetIFCEntity(MCObjectHandle hObject, const TXString& strIfcObjName) = 0;
			virtual VCOMError VCOM_CALLTYPE IFC_GetIFCEntity(MCObjectHandle hObject, TXString& strIfcObjName) = 0;
			virtual VCOMError VCOM_CALLTYPE IFC_SetEntityProp(MCObjectHandle hObject, const TXString& strIfcPropName, const TXString& strIfcPropValue) = 0;
			virtual VCOMError VCOM_CALLTYPE IFC_GetEntityProp(MCObjectHandle hObject, const TXString& strIfcPropName, TXString& strIfcPropValue, short& iType) = 0;
			virtual VCOMError VCOM_CALLTYPE IFC_GetNumPsets(MCObjectHandle hObject, short& numPsets) = 0;
			virtual VCOMError VCOM_CALLTYPE IFC_GetPsetName(MCObjectHandle hObject, short index, TXString& strIfcPsetName) = 0;
			virtual VCOMError VCOM_CALLTYPE IFC_AttachPset(MCObjectHandle hObject, const TXString& strIfcPsetName) = 0;
			virtual VCOMError VCOM_CALLTYPE IFC_SetPsetProp(MCObjectHandle hObject, const TXString& strIfcPsetName, const TXString& strIfcPropName, const TXString& strIfcPropValue) = 0;
			virtual VCOMError VCOM_CALLTYPE IFC_GetPsetProp(MCObjectHandle hObject, const TXString& strIfcPsetName, const TXString& strIfcPropName, TXString& strIfcPropValue, short& iType)= 0;
			virtual VCOMError VCOM_CALLTYPE IFC_ClearIFCInfo(MCObjectHandle hObject) = 0;
			virtual VCOMError VCOM_CALLTYPE IFC_SetEntityColor(const TXString& strIfcName, short red, short green, short blue, short transparent) = 0;
			virtual VCOMError VCOM_CALLTYPE IFC_GetEntityColor(const TXString& strIfcName, short& red, short& green, short& blue, short& transparent) = 0;
			virtual VCOMError VCOM_CALLTYPE IFC_CopyIFCData(MCObjectHandle hSource, MCObjectHandle hDestination, short inMode) = 0;
			virtual VCOMError VCOM_CALLTYPE GetIFCNameForSelection(TXString& outName, MCObjectHandle hSelectedObject = NULL) = 0;
			virtual VCOMError VCOM_CALLTYPE ShowIFCDataForSelection( MCObjectHandle hSelectedObject = NULL ) = 0;
			virtual VCOMError VCOM_CALLTYPE GetIFCProperty(MCObjectHandle hObject, const TXString& instanceProperty, TXString& outName) = 0;
			virtual VCOMError VCOM_CALLTYPE GetIFCName( MCObjectHandle hObject, TXString& outName ) = 0;
			virtual VCOMError VCOM_CALLTYPE GetAllIFCNamesInDocument( TXStringArray& outArrNames ) = 0;
			virtual	VCOMError VCOM_CALLTYPE IFC_ImportLibrary(IFolderIdentifier* pFolderID, bool bKeepHierarchy) = 0;

		};
	}
}


