//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License
//


#pragma once

namespace VWFC
{
	namespace PluginSupport
	{
		using namespace VectorWorks::Extension;

		// ------------------------------------------------------------------------------------------------------
		class VWMenu_EventSink	: public VCOMImpl<ILegacyEventSink>
		{
		public:
								VWMenu_EventSink(IVWUnknown* parent);
			virtual				~VWMenu_EventSink();

			void				SetUniversalName(const TXString& name);

		public:
			virtual void		DoInterface();
			virtual bool		GetItemEnabled();
			virtual void		OnDynamicVariableChunkUpdate();

		public:
			virtual long		GetUserData() const;
			virtual void		SetUserData(long userData);
			long				GetResult() const;
			void				SetResult(long result);
			void				SetMenuChunkInfo(MenuChunkInfo* pMenuChunkInfo);

			// The default event notificaiotn
			// if no virutal is present for an event, this function is called to allow the user to catch everything
			virtual long		OnDefaultEvent(long action, long message);

		// ILegacyEventSink
		public:
			virtual long VCOM_CALLTYPE			LegacyMain(long action, long message1, long message2);

		protected:
			TXString			fUnivName;
			MenuChunkInfo*		fpMenuChunkInfo;
			long				fResult;
		};

		// ------------------------------------------------------------------------------------------------------
		struct SMenuDef
		{
			Uint16			fNeeds;
			Uint16			fNeedsNot;
			SSTRResource	fTitle;
			SSTRResource	fCategory;
			STEXTResource	fHelpText;
			Sint16			fVersionCreated;
			Sint16			fVersoinModified;
			Sint16			fVersoinRetired;
			const char*		fContextualHelpID;
		};

		// ------------------------------------------------------------------------------------------------------
		struct SMenuChunkDef
		{
			SSTRResource	fItemText;
			STEXTResource	fHelpText;
			const char*		fContextualHelpID;
		};

		// ------------------------------------------------------------------------------------------------------
		class VWExtensionMenu :	public VCOMImmediateImpl<IExtensionMenu>
		{
		public:
											VWExtensionMenu(	CallBackPtr cbp,
																const SMenuDef& menuDef,
																const SMenuChunkDef* arrMenuChunks=NULL,
																EMenuChunkType chunkType=eMenuChunkType_Regular);
			virtual							~VWExtensionMenu();

		// IExtensionMenu
		public:
			virtual const TXString& VCOM_CALLTYPE	GetTitle();
			virtual const TXString& VCOM_CALLTYPE	GetCategory();
			virtual const TXString& VCOM_CALLTYPE	GetHelpText();
			virtual const TXString& VCOM_CALLTYPE	GetContextualHelpID(size_t chunkIndex = size_t(-1));
			virtual	void VCOM_CALLTYPE				GetNeedsFlags(Uint16& outNeeds, Uint16& outNeedsNot);
			virtual	void VCOM_CALLTYPE				GetVersionInfo(Sint16& outCreated, Sint16& outModified, Sint16& outRetired);

		// IExtension
		public:
			virtual IEventSink* VCOM_CALLTYPE		QueryEventSink(const TSinkIID& iid);

		protected:
			virtual void						OnRefCountZero();
			virtual ILegacyEventSink*			CreateLegacyEventSink(IVWUnknown* parent) = 0;
			virtual bool						GetProtectionSinkIID(TSinkIID& outIID);
			virtual IProviderProtection*		CreateProtectionSink(IVWUnknown* parent);

		protected:
			struct SChunkInfo {
				TXString	fItem;
				TXString	fContextualHelpID;
				TXString	fHelpText;
			};

			typedef std::vector<SChunkInfo>	TSubItemsVector;
			void						GetSubMenuItems(TSubItemsVector& outArrItems);
			void						SetSubMenuItems(const TSubItemsVector& arrItems);

		private:
			CallBackPtr						fCBP;

			const SMenuDef&					fMenuDef;
			TXString						fstrTitle;
			TXString						fstrHelp;
			TXString						fstrCategory;
			TXString						fstrContextualHelpID;
			EMenuChunkType					fMenuChunkType;
			bool							fMenuChunkIsDirty;

			TSubItemsVector					farrSubItems;

			class CMenuChunk;

			ILegacyEventSink*				fLegacyEventSink;
			CMenuChunk*						fMenuChunkSink;
			bool							fProtectionObtained;
			bool							fHasProtection;
			TSinkIID						fProtectionSinkIID;
			IProviderProtection*			fProtectionSink;

		private:
			// -------------------------------------------------------------------
			class CMenuChunk : public VCOMImpl<IMenuChunk2Provider>
			{
			public:
						CMenuChunk(VWExtensionMenu* parent);
				virtual	~CMenuChunk();

			// IMenuChunk2Provider
			public:
				virtual EMenuChunkType VCOM_CALLTYPE	GetType();
				virtual const TXString& VCOM_CALLTYPE	GetItemHelp(size_t index);
				virtual bool VCOM_CALLTYPE				IsChunkDirty();
				virtual void VCOM_CALLTYPE				ClearChunkDirty();

			// IMenuChunkProvider
			public:
				virtual size_t VCOM_CALLTYPE			GetItemsCount();
				virtual const TXString& VCOM_CALLTYPE	GetItemTitle(size_t index);
				virtual const TXString& VCOM_CALLTYPE	GetContextualHelpID(size_t index);
 
			private:
				VWExtensionMenu*		fParent;
			};
		};
	}
}
