//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License
//

#pragma once


namespace VWFC
{
	namespace Tools
	{
		// ---------------------------------------------------------------------------------------------------
		// STR# resource loader
		// Overcommes the problem of slow work of TXString::Load (or GS_GetResourceString)
		// This is singleton class that works on the current resource
		// ---------------------------------------------------------------------------------------------------
		class CSTRResLoader
		{
		protected:
									CSTRResLoader();

		public:
			virtual					~CSTRResLoader();

			static CSTRResLoader&	Instance();

		public:
			bool					CacheSTR(short id, bool useOnlyLocalResources = true);
			bool					Load(short id, short index, TXString& outString, bool useOnlyLocalResources = true);
			const TXString&			Load(short id, short index, bool useOnlyLocalResources = true);
			bool					LoadText(short id, TXString& outString, bool useOnlyLocalResources = true);
			TXString				LoadText(short id, bool useOnlyLocalResources = true);

			Uint16					GetStrResourceCount( short id );

		private:

			TXString				fInternalStringForLoad;
			TXString				fErrorReturnString;

			struct TStrResData {
				std::vector<unsigned char> fData;
				std::vector<const unsigned char *> fIndex;
			};

			typedef std::map<short, TStrResData>	TStringsMap;
			typedef std::pair<short, short>			TShortPair;
			typedef std::map<TShortPair, TXString>	TLoadedTXStringMap;

			TStringsMap								fmapStrings;
			TLoadedTXStringMap						fLoadedStrings;
		};
	}
}
