//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License
//

#ifndef VECTORWORKS_VIEWPORT_OBJECT__H
#define VECTORWORKS_VIEWPORT_OBJECT__H

#pragma once

namespace VWFC
{
	using namespace VectorWorks;

	namespace VWObjects
	{
		enum EProjectionType {
			eOrthogonal = 0,
			ePerspective = 1,
			eCavalierOblique45 = 2,
			eCavalierOblique30 = 3,
			eCabinetOblique45 = 4,
			eCabinetOblique30 = 5,
			ePlan = 6,
		};

		enum ERenderType {
			eRenderWireFrame = 0,
			eRenderQuick = 1,
			eRenderSolid = 2,
			eRenderShadedSolid = 3,
			eRenderShadedNoLines = 4,
			eRenderFinalShaded = 5,
			eRenderFinalHiddenLine = 6,
			eRenderDashedHiddenLine = 7,
			eRenderOpenGL = 11,
			eRenderFastRenderWorks = 12,
			eRenderFastShadowRenderWorks = 13,
			eRenderFinalRenderWorks = 14,
			eRenderCustomRenderWorks = 15,
			eRenderArtisticRenderWorks = 16,
			eRenderFastGI = 20,
			eRenderFinalGI = 21,
			eRenderCustomGI = 22,
			eRenderSketch = 18,
		};

		enum EViewType {
			eViewFront = 3,
			eViewBack = 4,
			eViewLeft = 5,
			eViewRight = 6,
			eViewTop = 7,
			eViewBottom = 8,
			eViewRightIso = 9,
			eViewLeftIso = 10,
			eViewRightRearIso = 11,
			eViewLeftRearIso = 12,
			eViewBottomRightIso = 13,
			eViewBottomLeftIso = 14,
			eViewBottomRightRearIso = 15,
			eViewBottomLeftRearIso = 16,
		};

		enum EVisibilityType {
			eInvisible = -1,
			eNormal = 0,
			eGrayed = 2,
		};

		class VWViewportObj : public VWObject
		{
		public:
								VWViewportObj(MCObjectHandle hViewport);
			static bool			CreateViewport(MCObjectHandle parentHandle, VWViewportObj& outViewportObj);
			virtual				~VWViewportObj();

			VWViewportObj&		operator=(const VWViewportObj& src);

		public:
			void				Update();
			bool				AddAnnotationObject(MCObjectHandle annotationHandle);
			bool				HasCropObject() const;
			bool				GetCropObject(MCObjectHandle& outCropHandle) const;
			bool				SetCropObject(MCObjectHandle inCropHandle);
			bool				GetCropVisible() const;
			void				SetCropVisible(bool cropVisible);
			bool				GetGroup(short groupType, MCObjectHandle& outGroupHandle) const;
			WorldPt				GetPosition() const;
			Real64				GetAngleToX() const;
			TXString			GetDescription() const;
			void				SetDescription(TXString description);
			TXString			GetLocator() const;
			void				SetLocator(TXString locator);
			bool				IsLinked() const;
			bool				GetLayerHeightIgnored() const;

			EProjectionType		GetProjectionType() const;
			void				SetProjectionType(EProjectionType projectionType);
			Real64				GetPerspectiveDistance() const;
			void				SetPerspectiveDistance(Real64 distance);
			bool				GetProject2D() const;
			void				SetProject2D(bool project2D);
			EViewType			GetViewType() const;
			void				SetViewType(EViewType viewType);
			bool				HasCamera() const;

			ERenderType			GetRenderType() const;
			void				SetRenderType(ERenderType renderType);
			ERenderType			GetForegroundRenderType() const;
			void				SetForegroundRenderType(ERenderType renderType);
			bool				IsDirty() const;
			InternalIndex		GetRenderBackground() const;
			void				SetRenderBackground(InternalIndex background);
			bool				GetAmbientIsOn() const;
			void				SetAmbientIsOn(bool isOn);
			Real64				GetAmbientBrightness() const;
			void				SetAmbientBrightness(Real64 brightness);
			WorldPt3			GetAmbientColor() const;
			void				SetAmbientColor(WorldPt3 color);
			InternalIndex		GetHDRIBackground() const;
			bool				GetShowWalls() const;
			void				SetShowWalls(bool showWalls);
			bool				GetGrayAsTransparent() const;
			void				SetGrayAsTransparent(bool grayAsTransparent);
			bool				GetFlipText() const;
			void				SetFlipText(bool flipText);
			bool				GetBlackAndWhite() const;
			void				SetBlackAndWhite(bool blackAndWhite);
			bool				GetDisplayPlanar() const;
			void				SetDisplayPlanar(bool displayPlanar);

			InternalIndex		GetSection2DClass() const;
			InternalIndex		GetSection2DOtherClass() const;
			InternalIndex		GetSection3DFillClass() const;
			InternalIndex		GetSection3DLineClass() const;

			Real64				GetScale() const;
			void				SetScale(Real64 scale);
			Real64				GetLineWeightScale() const;
			void				SetLineWeightScale(Real64 scale);
			Real64				GetArrowheadScale() const;
			void				SetArrowheadScale(Real64 scale);
			Real64				GetDashedLineScale() const;
			void				SetDashedLineScale(Real64 scale);
			Real64				GetHatchLineScale() const;
			void				SetHatchLineScale(Real64 scale);
			Real64				GetDesignTextScale() const;
			void				SetDesignTextScale(Real64 scale);
			Real64				GetSlashThicknessScale() const;
			void				SetSlashThicknessScale(Real64 scale);

			bool				GetUseDocumentClassVis() const;
			void				SetUseDocumentClassVis(bool useDocumentClassVis);
			bool				GetClassVisibility(InternalIndex classIndex, short& outVisibilityType) const;
			bool				SetClassVisibility(InternalIndex classIndex, short inVisibilityType);
			size_t				GetNumClassOverrides() const;
			InternalIndex		GetClassOverrideClass(size_t index) const;
			bool				CreateClassOverride(InternalIndex classIndex);
			bool				GetClassOverride(InternalIndex classIndex, SViewportClassOverride& outOverride) const;
			bool				SetClassOverride(InternalIndex classIndex, const SViewportClassOverride& inOverride);
			bool				RemoveClassOverride(InternalIndex classIndex);

			size_t				GetNumLightOverrides() const;
			InternalIndex		GetLightOverrideLight(size_t index) const;
			bool				CreateLightOverride(InternalIndex lightIndex);
			bool				GetLightOverride(InternalIndex lightIndex, SViewportLightOverride& outOverride) const;
			bool				SetLightOverride(InternalIndex lightIndex, const SViewportLightOverride& inOverride);
			bool				RemoveLightOverride(InternalIndex lightIndex);

			bool				GetLayerVisibility(InternalIndex layerIndex, short& outVisibilityType) const;
			bool				SetLayerVisibility(InternalIndex layerIndex, short inVisibilityType);
			size_t				GetNumLayerOverrides() const;
			InternalIndex		GetLayerOverrideLayer(size_t index) const;
			bool				CreateLayerOverride(InternalIndex layerIndex);
			bool				GetLayerOverride(InternalIndex layerIndex, SViewportLayerOverride& outOverride) const;
			bool				SetLayerOverride(InternalIndex layerIndex, const SViewportLayerOverride& inOverride);
			bool				RemoveLayerOverride(InternalIndex layerIndex);

			size_t				GetNumLayerPropertiesOverrides() const;
			InternalIndex		GetLayerPropertiesOverrideLayer(size_t index) const;
			bool				CreateLayerPropertiesOverride(InternalIndex layerIndex);
			bool				GetLayerPropertiesOverride(InternalIndex layerIndex, SViewportLayerPropertiesOverride& outOverride) const;
			bool				SetLayerPropertiesOverride(InternalIndex layerIndex, const SViewportLayerPropertiesOverride& inOverride);
			bool				RemoveLayerPropertiesOverride(InternalIndex layerIndex);

			size_t				GetNumLayerStackingOverrides() const;
			InternalIndex		GetLayerStackingOverrideLayer(size_t index) const;
			bool				GetLayerStackingOverride(InternalIndex layerIndex, long& outStackingPosition) const;
			bool				SetLayerStackingOverride(InternalIndex layerIndex, long inStackingPosition);
			bool				RemoveLayerStackingOverride(InternalIndex layerIndex);

		public:
			static bool			IsViewportObject(MCObjectHandle hObj);
			static bool			IsDesignLayerViewportObject(MCObjectHandle hObj);
		};
	}
}

#endif