//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License
//

#pragma once

namespace VWFC
{
	namespace VWUI
	{
		class VWDialog;

		// ---------------------------------------------------------------------------------------------------------------
		enum EListBrowserEventType {
			kListBrowserEventType_Unknown						= 0,
			kListBrowserEventType_DataChangeClick				= -2,
			kListBrowserEventType_DataChangeAllClick			= -3,
			kListBrowserEventType_SelectionChangeClick			= -4,
			kListBrowserEventType_DoubleClick					= -5,
			kListBrowserEventType_DoubleAllClick				= -14,
			kListBrowserEventType_DoubleRecursiveClick			= -15,
			kListBrowserEventType_DeleteKeyPressed				= -6,
			kListBrowserEventType_UpKeyPressed					= -7,
			kListBrowserEventType_DownKeyPressed				= -8,
			kListBrowserEventType_AlphaNumericKeyPressed		= -9,
			kListBrowserEventType_SortOccurred					= -10,
			kListBrowserEventType_SelectionMultChangeClick		= -11,
			kListBrowserEventType_EnterKeyPressed				= -12,
			kListBrowserEventType_DataChangeRecursiveClick		= -13,
			kListBrowserEventType_DragAndDrop					= -51,
		};

		enum EEditFieldEventType 
		{
			kEditFieldEventType_TextChanged		= 0,
			kEditFieldEventType_Focused			= 1
		};

		
		// ---------------------------------------------------------------------------------------------------------------
		class VWDialogEventArgs
		{
			friend class VWDialog;
		protected:
									VWDialogEventArgs(long dlgID, long ctrlID, void* pData);
		public:
									VWDialogEventArgs();
			virtual					~VWDialogEventArgs();

		// general events support
		public:
			long					GetEventData() const;

		// list browser events support
		public:
			EListBrowserEventType	GetListBrowserEvent(size_t& outRowIndex, size_t& outColumnIndex) const;
			bool					IsListBrowserDoubleClickEvent() const;
			bool					IsListBrowserDrawNDropEnd() const;
			bool					IsListBrowserNextMessageIsDropInfo() const;

			struct SListBrowserDragDropInfo
			{
				std::vector<int>		vIndicesDragged;
				int						iDragIndex;
				int						iDropIndex;
				int						iSortState;

				SListBrowserDragDropInfo() { iDragIndex = iDropIndex = iSortState = -1; }
			};

			SListBrowserDragDropInfo*	GetListBrowserDropInfo();


		// pull down menu events support
		public:
			bool					IsPullDownNotification() const;

		// custom control events support
		public:
			bool					IsCustomCtrlMouseDown() const;
			bool					IsCustomCtrlClickEvent() const;
			bool					IsCustomCtrlDoubleClickEvent() const;
			bool					IsCustomCtrlRightClickEvent() const;
			bool					IsCustomCtrlRightDoubleClickEvent() const;
			bool					IsCustomCtrlMiddleClickEvent() const;
			bool					IsCustomCtrlMiddleDoubleClickEvent() const;
			long					GetCustomCtrlEventPosX() const;
			long					GetCustomCtrlEventPosY() const;

		public:
			long				fDialogID;
			long				fControlID;
			void*				fpData;
			bool				fbNegativeControlID;
		};
	}
}