//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License
//


#include "StdHeaders.h"
//#include "VWPolygon.h"


#include <MCMatrix.h>

using namespace VWFC::Math;

// ---------------------------------------------------------------------------------------------------------------------
VWPolygon2D::CLockForEdit::CLockForEdit(VWPolygon2D& poly)
	: fPoly( poly )
{
	fPoly.fLockedForEdit	= true;
}

VWPolygon2D::CLockForEdit::~CLockForEdit()
{
	fPoly.fLockedForEdit	= false;
	fPoly.RecalculateBoundingBox();
}

// ---------------------------------------------------------------------------------------------------------------------
VWPolygon2D::SVertex::SVertex()
{
	type			= eVertexType_Corner;
	dArcRadius		= 0.0;
	bVisible		= true;
}

VWPolygon2D::SVertex::SVertex(const SVertex& src)
{
	type		= src.type;
	pt			= src.pt;
	dArcRadius	= src.dArcRadius;
	bVisible	= src.bVisible;
}

VWPolygon2D::SVertex::~SVertex()
{
}

VWPolygon2D::SVertex& VWPolygon2D::SVertex::operator=(const VWPolygon2D::SVertex& src)
{
	type		= src.type;
	pt			= src.pt;
	dArcRadius	= src.dArcRadius;
	bVisible	= src.bVisible;

	return	*this;
}

bool VWPolygon2D::SVertex::operator==(const VWPolygon2D::SVertex& that) const
{
	return ( ( type			== that.type		) &&
			 ( pt			== that.pt			) &&
			 ( dArcRadius	== that.dArcRadius	) &&
			 ( bVisible		== that.bVisible	) );
}

bool VWPolygon2D::SVertex::operator!=(const VWPolygon2D::SVertex& that) const
{
	return ! ( this->operator == ( that ) );
}

// ---------------------------------------------------------------------------------------------------------------------
VWPolygon2D::SExtVertex::SExtVertex()
	: SVertex()
{
}

VWPolygon2D::SExtVertex::SExtVertex(const SExtVertex& src)
{
	type			= src.type;
	pt				= src.pt;
	dArcRadius		= src.dArcRadius;
	bVisible		= src.bVisible;
	controlPt		= src.controlPt;
}

VWPolygon2D::SExtVertex::SExtVertex(const SVertex& src)
{
	type			= src.type;
	pt				= src.pt;
	dArcRadius		= src.dArcRadius;
	bVisible		= src.bVisible;
}

VWPolygon2D::SExtVertex::~SExtVertex()
{
}

VWPolygon2D::SExtVertex& VWPolygon2D::SExtVertex::operator=(const VWPolygon2D::SExtVertex& src)
{
	type			= src.type;
	pt				= src.pt;
	dArcRadius		= src.dArcRadius;
	bVisible		= src.bVisible;
	controlPt		= src.controlPt;

	return	*this;
}

VWPolygon2D::SExtVertex& VWPolygon2D::SExtVertex::operator=(const VWPolygon2D::SVertex& src)
{
	type			= src.type;
	pt				= src.pt;
	dArcRadius		= src.dArcRadius;
	bVisible		= src.bVisible;

	return	*this;
}

bool VWPolygon2D::SExtVertex::operator==(const VWPolygon2D::SExtVertex& that) const
{
	return ( ( type			== that.type		) &&
			 ( pt			== that.pt			) &&
			 ( dArcRadius	== that.dArcRadius	) &&
			 ( bVisible		== that.bVisible	) &&
			 ( controlPt	== that.controlPt	) );
}

bool VWPolygon2D::SExtVertex::operator!=(const VWPolygon2D::SExtVertex& that) const
{
	return ! ( this->operator == ( that ) );
}

// ---------------------------------------------------------------------------------------------------------------------
VWPolygon2D::VWPolygon2D()
{
	fdMinX			= 0;
	fdMaxX			= 0;
	fdMinY			= 0;
	fdMaxY			= 0;
	fbIsClosed		= true;
	fLockedForEdit	= false;
}

VWPolygon2D::VWPolygon2D(const TSTLPoints2DArray& src)
{
	size_t		len		= src.size();
	for(size_t i=0; i<len; i++)
		this->AddVertex( src[ i ] );
	fbIsClosed		= false;
	fLockedForEdit	= false;
}

VWPolygon2D::VWPolygon2D(const TSTLPoints2DVWArray& src)
{
	size_t		len		= src.size();
	for(size_t i=0; i<len; i++)
		this->AddVertex( src[ i ] );

	fbIsClosed		= false;
	fLockedForEdit	= false;
}

VWPolygon2D::VWPolygon2D(IPolyDefPtr pPolygonDef)
{
	fdMinX			= 0;
	fdMaxX			= 0;
	fdMinY			= 0;
	fdMaxY			= 0;
	fbIsClosed		= true;
	fLockedForEdit	= false;

	this->operator = ( pPolygonDef );
}

VWPolygon2D::VWPolygon2D(const VWPolygon2D& src)
{
	fdMinX				= src.fdMinX;
	fdMaxX				= src.fdMaxX;
	fdMinY				= src.fdMinY;
	fdMaxY				= src.fdMaxY;
	fbIsClosed			= src.fbIsClosed;
	fLockedForEdit		= src.fLockedForEdit;

	size_t		len		= src.farrVertices.size();
	for(size_t i=0; i<len; i++)
		farrVertices.push_back( src.farrVertices[ i ] );
}


VWPolygon2D::~VWPolygon2D()
{
	farrVertices.clear();
}

VWPolygon2D& VWPolygon2D::operator=(const TSTLPoints2DArray& src)
{
	this->ClearVertices();
	size_t		len		= src.size();
	for(size_t i=0; i<len; i++)
		this->AddVertex( src[ i ] );
	return *this;
}

VWPolygon2D& VWPolygon2D::operator=(const TSTLPoints2DVWArray& src)
{
	this->ClearVertices();
	size_t		len		= src.size();
	for(size_t i=0; i<len; i++)
		this->AddVertex( src[ i ] );
	return *this;
}

VWPolygon2D& VWPolygon2D::operator=(IPolyDefPtr pPolygonDef)
{
	this->ClearVertices();
	if ( pPolygonDef ) {
		Uint32	cnt		= 0;
		pPolygonDef->GetVertexCount( cnt );
		for(Uint32 i=0; i<cnt; i++) {
			WorldPt		pt;
			EVertexType	vertType	= eVertexType_Corner;
			double		dRadius		= 0;

			pPolygonDef->GetVertexAt( i, pt );
			pPolygonDef->GetVertexType( i, vertType );
			pPolygonDef->GetArcVertexRadius( i, dRadius );
			bool	bVisible	= VCOM_SUCCEEDED( pPolygonDef->GetVertexVisible( i ) );

			SVertex		vertex;
			vertex.type				= vertType;
			vertex.pt.x				= pt.x;
			vertex.pt.y				= pt.y;
			vertex.dArcRadius		= dRadius;
			vertex.bVisible			= bVisible;
			farrVertices.push_back( vertex );
		}

		this->RecalculateBoundingBox();
	}
	return *this;
}

VWPolygon2D& VWPolygon2D::operator=(const VWPolygon2D& src)
{
	fdMinX				= src.fdMinX;
	fdMaxX				= src.fdMaxX;
	fdMinY				= src.fdMinY;
	fdMaxY				= src.fdMaxY;
	fbIsClosed			= src.fbIsClosed;
	
	farrVertices.clear();
	size_t		len		= src.farrVertices.size();
	for(size_t i=0; i<len; i++)
		farrVertices.push_back( src.farrVertices[ i ] );

	return *this;
}

void VWPolygon2D::SetPolygon(const TSTLPoints2DArray& src)
{
	(*this) = src;
}

void VWPolygon2D::SetPolygon(const TSTLPoints2DVWArray& src)
{
	(*this) = src;
}

void VWPolygon2D::SetPolygon(IPolyDefPtr pPolygonDef)
{
	(*this) = pPolygonDef;
}

void VWPolygon2D::GetPolygon(TSTLPoints2DArray& out) const
{
	out.clear();
	size_t		len		= farrVertices.size();
	for(size_t i=0; i<len; i++)
		out.push_back( farrVertices[i].pt );
}

void VWPolygon2D::GetPolygon(TSTLPoints2DVWArray& out) const
{
	out.clear();
	size_t		len		= farrVertices.size();
	for(size_t i=0; i<len; i++)
		out.push_back( farrVertices[i].pt );
}

void VWPolygon2D::GetPolygon(IPolyDefPtr pPolygonDef) const
{
	if ( pPolygonDef ) {
		pPolygonDef->ClearVertices();
		size_t		len		= farrVertices.size();
		for(size_t i=0; i<len; i++) {
			const SVertex&	vertex = farrVertices[ i ];

			if ( vertex.type == eVertexType_Arc ) {
				pPolygonDef->AddVertex( vertex.pt, vertex.dArcRadius );
			}
			else {
				pPolygonDef->AddVertex( vertex.pt, vertex.type );
			}
		}
	}
}

void VWPolygon2D::ClearVertices()
{
	fdMinX			= 0;
	fdMaxX			= 0;
	fdMinY			= 0;
	fdMaxY			= 0;
	farrVertices.clear();
	fbIsClosed		= true;
}

void VWPolygon2D::AddVertex(const VWPoint2D& pt, EVertexType type /*=eVertexType_Corner*/, double dArcRadius /*=0*/)
{
	SVertex		vertex;
	vertex.type				= type;
	vertex.pt				= pt;
	vertex.dArcRadius		= dArcRadius;
	vertex.bVisible			= true;
	farrVertices.push_back( vertex );

	if ( ! fLockedForEdit )
	{
		this->RecalculateBoundingBox();
	}
}

void VWPolygon2D::AddVertex(double x, double y, EVertexType type /*=eVertexType_Corner*/, double dArcRadius /*=0*/)
{
	SVertex		vertex;
	vertex.type				= type;
	vertex.pt.x				= x;
	vertex.pt.y				= y;
	vertex.dArcRadius		= dArcRadius;
	vertex.bVisible			= true;
	farrVertices.push_back( vertex );

	if ( ! fLockedForEdit )
	{
		this->RecalculateBoundingBox();
	}
}

void VWPolygon2D::AddVertex(const VWPolygon2D& poly)
{
	size_t	vertsCnt	= poly.farrVertices.size();
	for(size_t i=0; i<vertsCnt; i++) {
		const SVertex&	vertex = poly.farrVertices[ i ];
		farrVertices.push_back( vertex );
	}

	if ( ! fLockedForEdit )
	{
		this->RecalculateBoundingBox();
	}
}

void VWPolygon2D::InsertVertex(size_t at, double x, double y, EVertexType type /*=eVertexType_Corner*/, double dArcRadius /*=0*/)
{
	SVertex		vertex;
	vertex.type				= type;
	vertex.pt.x				= x;
	vertex.pt.y				= y;
	vertex.dArcRadius		= dArcRadius;
	vertex.bVisible			= true;
	farrVertices.insert( farrVertices.begin() + at, vertex );

	if ( ! fLockedForEdit )
	{
		this->RecalculateBoundingBox();
	}
}

void VWPolygon2D::InsertVertex(size_t at, const VWPoint2D& pt, EVertexType type /*=eVertexType_Corner*/, double dArcRadius /*=0*/)
{
	SVertex		vertex;
	vertex.type				= type;
	vertex.pt				= pt;
	vertex.dArcRadius		= dArcRadius;
	vertex.bVisible			= true;
	farrVertices.insert( farrVertices.begin() + at, vertex );

	if ( ! fLockedForEdit )
	{
		this->RecalculateBoundingBox();
	}
}

void VWPolygon2D::InsertVertex(size_t at, const VWPolygon2D& poly)
{
	size_t	vertsCnt	= poly.farrVertices.size();
	for(size_t i=0; i<vertsCnt; i++) {
		const SVertex&	vertex = poly.farrVertices[ vertsCnt - i - 1 ];
		farrVertices.insert( farrVertices.begin() + at, vertex );
	}

	if ( ! fLockedForEdit )
	{
		this->RecalculateBoundingBox();
	}
}

void VWPolygon2D::ClearVertex(size_t at)
{
	farrVertices.erase( farrVertices.begin() + at );
	this->RecalculateBoundingBox();
}

size_t VWPolygon2D::GetVertexCount() const
{
	return farrVertices.size();
}

bool VWPolygon2D::IsEmpty() const
{
	return farrVertices.size() <= 1;
}

VWPoint2D& VWPolygon2D::GetVertexAt(size_t at, bool bLoopVerts /*=false*/)
{
	if ( bLoopVerts ) {
		// loop the vertex indeces inside the array
		VWFC_ASSERT( farrVertices.size() > 0 );
		at		= at % farrVertices.size();
	}
	else {
		VWFC_ASSERT( at < farrVertices.size() );
	}
	return farrVertices[ at ].pt;
}

const VWPoint2D& VWPolygon2D::GetVertexAt(size_t at, bool bLoopVerts /*=false*/) const
{
	if ( bLoopVerts ) {
		// loop the vertex indeces inside the array
		VWFC_ASSERT( farrVertices.size() > 0 );
		at		= at % farrVertices.size();
	}
	else {
		VWFC_ASSERT( at < farrVertices.size() );
	}
	return farrVertices[ at ].pt;
}

EVertexType VWPolygon2D::GetVertexType(size_t at) const
{
	VWFC_ASSERT( at < farrVertices.size() );
	return farrVertices[ at ].type;
}

void VWPolygon2D::SetVertexType(size_t at, EVertexType type)
{
	VWFC_ASSERT( at < farrVertices.size() );
	farrVertices[ at ].type = type;
}

double VWPolygon2D::GetArcVertexRadius(size_t at) const
{
	VWFC_ASSERT( at < farrVertices.size() );
	VWFC_ASSERT( farrVertices[ at ].type == eVertexType_Arc );
	return farrVertices[ at ].dArcRadius;
}

void VWPolygon2D::SetArcVertexRadius(size_t at, double dRadius)
{
	VWFC_ASSERT( at < farrVertices.size() );
	VWFC_ASSERT( farrVertices[ at ].type == eVertexType_Arc );
	farrVertices[ at ].dArcRadius	= dRadius;
}

bool VWPolygon2D::GetVertexVisible(size_t at) const
{
	VWFC_ASSERT( at < farrVertices.size() );
	return farrVertices[ at ].bVisible;
}

void VWPolygon2D::SetVertexVisible(size_t at, bool bVisible)
{
	VWFC_ASSERT( at < farrVertices.size() );
	farrVertices[ at ].bVisible		= bVisible;
}

// return a line from (at, at+1) vertex (at the end the line is (len-1, 0)
VWLine2D VWPolygon2D::GetLineAt(size_t at) const
{
	VWFC_ASSERT( at < farrVertices.size() );
	size_t	next	= at + 1;
	if ( next >= farrVertices.size() )
		next	= 0;

	return VWLine2D( farrVertices[ at ].pt, farrVertices[ next ].pt );
}

void VWPolygon2D::RecalculateBoundingBox()
{
	fdMinX		= 0;
	fdMaxX		= 0;
	fdMinY		= 0;
	fdMaxY		= 0;

	size_t	len	= farrVertices.size();
	if ( len > 0 )
	{
		const	VWPoint2D&	pt	= farrVertices[ 0 ].pt;
		fdMinX		= pt.x;
		fdMaxX		= pt.x;
		fdMinY		= pt.y;
		fdMaxY		= pt.y;

		for(size_t i=1; i<len; i++)
		{
			const	VWPoint2D&	pt	= farrVertices[ i ].pt;
			fdMinX		= min( fdMinX, pt.x );
			fdMaxX		= max( fdMaxX, pt.x );
			fdMinY		= min( fdMinY, pt.y );
			fdMaxY		= max( fdMaxY, pt.y );
		}
	}
}

void VWPolygon2D::SetClosed(bool isClosed)
{
	fbIsClosed		= isClosed;
	size_t		len	= farrVertices.size();
	if ( len > 0 )
	{
		farrVertices[ len-1 ].bVisible	= isClosed;
	}
}

bool VWPolygon2D::IsClosed() const
{
	return fbIsClosed;
}

void VWPolygon2D::SetAllVertexVisible(bool bVisible)
{
	size_t	len	= farrVertices.size();
	for(size_t i=0; i<len; i++) {
		farrVertices[ i ].bVisible		= bVisible;
	}
}

void VWPolygon2D::GetPolygonBounds(double& outMinX, double& outMaxX, double& outMinY, double& outMaxY) const
{
	outMinX		= fdMinX;
	outMaxX		= fdMaxX;
	outMinY		= fdMinY;
	outMaxY		= fdMaxY;
}

VWRectangle2D VWPolygon2D::GetPolygonBounds() const
{
	return VWRectangle2D( fdMinX, fdMaxX, fdMaxY, fdMinY );
}

VWPoint2D VWPolygon2D::GetPolygonBoundsCenter() const
{
	return VWPoint2D( (fdMinX+fdMaxX)/2, (fdMinY+fdMaxY)/2 );
}

// reverse the order of the points (if CW to CCW or CCW to CW)
void VWPolygon2D::ReversePoints(const bool& kbDoFullReversion  /* = false */)
{
 	size_t	len	= farrVertices.size();
	for(size_t i=0; i<len / 2; i++)
	{
		/*
			Get the vertices to be exchanged as references, so that to avoid the multiple
			use of the array access operator. A problem could arise if copies of the vertices
			are generated and modified instead of editing the elements of the farrVertices STL vector.
			Actually, STL vector's operator[] returns either a modifiable reference or a constant
			reference to the visited element. Therefore, it is guaranteed that the following expression:
			"SVertex& leftVertex = farrVertices[ i ];" won't make the compiler copy the vertex
			"farrVertices[ i ]" to a temporary implicit ( hidden ) variable of SVertex type
			and point the reference "SVertex& leftVertex" to it.
			Another possible flaw when using references to dynamically allocated data
			would occur if the it is expanded or diminished e.g. the reserved memory
			for the data is reallocated. Then there is a great chance to lose the reference.
			This case is impossible to be encountered, since we are only modifying the data
			e.g. what farrVertices points to, not adding or removing data elements
			( changing its size thus changing the value held by farrVertices ).
		*/
		SVertex&		leftVertex 			= farrVertices[ i ];
		VWPoint2D		temp				= leftVertex.pt;
		const size_t	knRghtVertexIndex	= len-i-1;
		SVertex&		rghtVertex 			= farrVertices[ knRghtVertexIndex ];
		leftVertex.pt						= rghtVertex.pt;
		rghtVertex.pt						= temp;

		if (kbDoFullReversion) {
			// Obtain the additional data from the LEFT vertex.
			const EVertexType	kLeftVertexType			= leftVertex.type;
			const double		kdLeftVertexArcRadius	= leftVertex.dArcRadius;
			const bool			kbLeftVertexIsVisible	= leftVertex.bVisible;

			// Copy the data of the RIGHT vertex to that of the LEFT one.
			leftVertex.type			= rghtVertex.type;
			leftVertex.dArcRadius	= rghtVertex.dArcRadius;
			leftVertex.bVisible		= rghtVertex.bVisible;

			// Set the data of the RIGHT vertex with that, which has already been
			// gathered above from the LEFT vertex.
			rghtVertex.type			= kLeftVertexType;
			rghtVertex.dArcRadius	= kdLeftVertexArcRadius;
			rghtVertex.bVisible		= kbLeftVertexIsVisible;
		}
	}

	if ( ! fLockedForEdit )
	{
		this->RecalculateBoundingBox();
	}
}

// return true if the passed polygon has more than 3 different vertices
bool VWPolygon2D::IsValidPolygon(double dEpsilon) const
{
	size_t	len	= farrVertices.size();
	if ( len < 3 )
		return false;

	size_t		count	= 1;
	VWPoint2D	pt		= farrVertices[ 0 ].pt;
	for(size_t i=1; i<len; i++)
	{
		double_gs	dist	= ( pt - farrVertices[ i ].pt ).Magnitude();
		if ( dist > dEpsilon )
		{
			count++;
			if ( count >= 3 )
				return true;
		}

		pt			= farrVertices[ i ].pt;
	}

	return count >= 3;
}

// removed all same points in the polygon
void VWPolygon2D::CleanUpPolygon(double dEpsilon)
{
	VWPoint2D					lastPt;
	TVertexArray::iterator		it		= farrVertices.begin();
	while( it != farrVertices.end() )
	{
		if ( it == farrVertices.begin() ) 
		{
			lastPt		= it->pt;
			++it;
		}
		else
		{
			double_gs	dist	= ( lastPt - it->pt ).Magnitude();
			if ( dist < dEpsilon )
			{
				// remove the unnecessary point
				farrVertices.erase( it );
				// start all over
				it		= farrVertices.begin();
			}
			else {
				lastPt		= it->pt;
				++it;
			}
		}
	}

	if ( ! fLockedForEdit )
	{
		this->RecalculateBoundingBox();
	}
}

// This function works as follows depending on 'doClosenessCheck':
// ( doClosenessCheck == false )	- clean up all polygon vertices which are collinear with their neighbors.
// ( doClosenessCheck == true  )	- clean up all polygon vertices which are collinear with their neighbors and are too close to their neighbors.
// Note: works on polygon with only corner vertices.
void VWPolygon2D::CleanUp_FeaturePreserving(bool doClosenessCheck, double epsCollinear, double epsCloseness)
{
	size_t					vertCnt		= this->GetVertexCount();
	bool					isClosed	= this->IsClosed();

	size_t					procCnt		= vertCnt;
	size_t					startFrom	= 0;
	if ( ! isClosed )
	{
		-- procCnt;
		++ startFrom;
	}

	// loop polygon vertices.
	size_t					i			= startFrom;
	while ( i < procCnt )
	{
		size_t				thisInx		= i;	// this index.

		size_t				prevInx;	// index of previous poly point.
		if ( isClosed )		prevInx		= ( thisInx > 0 ) ? ( thisInx - 1 ) : ( procCnt - 1 );	// note that ( vertCnt == procCnt ) here !
		else				prevInx		= ( thisInx - 1 );										// note that ( thisInx > 0 ) here !

		size_t				nextInx;	// index of next poly point.
		if ( isClosed )		nextInx		= ( thisInx + 1 ) % procCnt;
		else				nextInx		= ( thisInx + 1 );

		const VWPoint2D&	thisPt		= this->GetVertexAt( thisInx );
		const VWPoint2D&	prevPt		= this->GetVertexAt( prevInx );
		const VWPoint2D&	nextPt		= this->GetVertexAt( nextInx );

		bool				doRemove	= false;

		const VWPoint2D&	vecToPrev	= ( prevPt - thisPt ).Normalize();
		const VWPoint2D&	vecToNext	= ( nextPt - thisPt ).Normalize();

		// check if the three points lie on a line.
		if ( MathUtils::Equalish( vecToPrev.CrossProductMagnitude( vecToNext ), 0.0, epsCollinear ) ||
			 MathUtils::Equalish( vecToPrev.CrossProductMagnitude( -vecToNext ), 0.0, epsCollinear ) )
		{
			if ( doClosenessCheck )
			{
				if ( ( thisPt.DistanceTo( prevPt ) < epsCloseness ) ||		// distance to previous vertex is too small.
					 ( thisPt.DistanceTo( nextPt ) < epsCloseness ) )		// distance to next vertex is too small.
				{
					doRemove			= true;
				}
			}
			else
			{
				doRemove				= true;
			}
		}

		if ( doRemove )
		{
			farrVertices.erase( farrVertices.begin() + i );
			-- procCnt;
		}
		else
		{
			++ i;						// increase counter.
		}
	}

	if ( ! fLockedForEdit )
	{
		this->RecalculateBoundingBox();
	}
}

// check polygon for self intersections. return true is so.
// 'doConsiderTouching' tells if touching will be considered as intersection.
bool VWPolygon2D::IsSelfIntersecting(bool doConsiderTouching/*=true*/) const
{
	size_t					ptsCnt		= this->GetVertexCount();

	// prevent check at the end of the polygon (when first and last points coincide).
	// don't check the last legment if this is open poly.
	if ( ptsCnt > 0 )
	{
		const VWPoint2D&	firstPt		= this->GetVertexAt( 0 );
		const VWPoint2D&	lastPt		= this->GetVertexAt( ptsCnt - 1 );

		if ( MathUtils::Equalish( firstPt, lastPt, VWPoint2D::sEpsilon ) ||		// ends coincide.
			 ! this->IsClosed() )												// poly is open.
		{
			-- ptsCnt;
		}
	}

	VWPoint2D				intersPt;
	bool			bSelfIntersection	= false;
	for ( size_t i = 0 ; ( i < ptsCnt ) && ( ! bSelfIntersection ) ; i ++ ) {

		size_t				prevInx		= ( i > 0 ) ? i - 1 : ptsCnt - 1;
		size_t				nextInx		= ( i + 1 ) % ptsCnt;

		const VWPoint2D&	pt1line1	= this->GetVertexAt( i );
		const VWPoint2D&	pt2line1	= this->GetVertexAt( nextInx );

		// first line.
		const VWLine2D		line1( pt1line1, pt2line1 );

		for ( size_t j = 0 ; j < ptsCnt ; j ++ ) {

			// do not check for same edge or its adjacent.
			if ( i != j && prevInx != j && nextInx != j ) {

				const VWPoint2D&	pt1line2	= this->GetVertexAt( j );
				const VWPoint2D&	pt2line2	= this->GetVertexAt( ( j + 1 ) % ptsCnt );

				// second line.
				const VWLine2D		line2( pt1line2, pt2line2 );

				if ( line2.IntersectLineSegments( line1, intersPt ) ) {

					// ensure that the intersection point lies on both lines (no epsilon!).
					bool			isOnLines	= false;

					double			lenLine		= VWPoint2D::DistanceSqare( pt1line1.x, pt1line1.y, pt2line1.x, pt2line1.y );
					double			distToPt1	= VWPoint2D::DistanceSqare( intersPt.x, intersPt.y, pt1line1.x, pt1line1.y );
					double			distToPt2	= VWPoint2D::DistanceSqare( intersPt.x, intersPt.y, pt2line1.x, pt2line1.y );

					bool			isOnLine	= ( doConsiderTouching ) ?
												  ( distToPt1 <= lenLine ) && ( distToPt2 <= lenLine ) :
												  ( distToPt1 <  lenLine ) && ( distToPt2 <  lenLine );

					if ( isOnLine ) {			// intersection lies on line 1 (no epsilon!).

						lenLine					= VWPoint2D::DistanceSqare( pt1line2.x, pt1line2.y, pt2line2.x, pt2line2.y );
						distToPt1				= VWPoint2D::DistanceSqare( intersPt.x, intersPt.y, pt1line2.x, pt1line2.y );
						distToPt2				= VWPoint2D::DistanceSqare( intersPt.x, intersPt.y, pt2line2.x, pt2line2.y );

						isOnLine				= ( doConsiderTouching ) ?
												  ( distToPt1 <= lenLine ) && ( distToPt2 <= lenLine ) :
												  ( distToPt1 <  lenLine ) && ( distToPt2 <  lenLine );

						if ( isOnLine ) {		// intersection lies on line 2 (no epsilon!).
							isOnLines			= true;
						}
					}

					if ( isOnLines ) {	// it is really a self-intersection point.
						bSelfIntersection	= true;
						break;
					}
				}
			}
		}
	}
	return bSelfIntersection;
}

// get the bouding box which is parallel to given line
// the result polygon contains 4 vertices of the bounding box
void VWPolygon2D::GetPolygonBounds(const VWLine2D& line, VWPolygon2D& outBounds) const
{
	VWPoint2D			basePt1		= line.GetStart();

	VWPoint2D			minPt, maxPt;
	VWPoint2D			minProjPt, maxProjPt;
	// min and max distance from the begining of the line (face bearing line)
	double_gs			minProjDist, maxProjDist;
	// mian am max dist from the poly to the line
	double_gs			minDist, maxDist;
	size_t				vertCnt		= farrVertices.size();
	for(size_t v=0; v<vertCnt; v++)
	{
		VWPoint2D	pt			= farrVertices[ v ].pt;
		VWPoint2D	projPt		= line.PtPerpLine( pt );
		double_gs	dist		= ( projPt - pt ).Magnitude();
		double_gs	projDist	= ( projPt - basePt1 ).Magnitude();
		if ( v == 0 )
		{
			// first time - init
			minPt			= maxPt				= pt;
			minProjPt		= maxProjPt			= projPt;
			minProjDist		= maxProjDist		= projDist;
			minDist			= maxDist			= dist;
		}
		else
		{
			if ( projDist < minProjDist )
			{
				minPt			= pt;
				minProjPt		= projPt;
				minProjDist		= projDist;
			}
			if ( projDist > maxProjDist )
			{
				maxPt			= pt;
				maxProjPt		= projPt;
				maxProjDist		= projDist;
			}
			if ( dist < minDist )
				minDist		= dist;
			if ( dist > maxDist )
				maxDist		= dist;
		}
	}

	VWPoint2D	minDir	= (minPt - minProjPt).Normalize();
	VWPoint2D	maxDir	= (maxPt - maxProjPt).Normalize();

	// calculate the four points of the rotated bounding box
	VWPoint2D	a		= minProjPt + minDir * minDist;
	VWPoint2D	b		= maxProjPt + minDir * minDist;
	VWPoint2D	c		= minProjPt + maxDir * maxDist;
	VWPoint2D	d		= maxProjPt + maxDir * maxDist;

	outBounds.ClearVertices();
	outBounds.AddVertex( a );
	outBounds.AddVertex( b );
	outBounds.AddVertex( d );
	outBounds.AddVertex( c );
}

// get the bounding box which is parallel to the given direction (a similar approach).
// the result is 4 vertices (UL, UR, LL, LR) of the bounding box
// XXX Bobi: in contrast to the previous, this one first polygonizes 
// the polygon and then gets the bounding box. It works for arbitrary polylines.
void VWPolygon2D::GetPolygonBounds(const VWPoint2D& vDirection, VWPoint2D& vUL, VWPoint2D& vUR, VWPoint2D& vLL, VWPoint2D& vLR) const
{
	// the direction have to be defined
	VWFC_ASSERT( vDirection.Magnitude() > VWPoint2D::sEpsilon );

	// get polygonized polygon
	VWPolygon2D			poly2DPolygonized;
	VWPolygon2DObj		poly2DObj( ::GS_ConvertToPolygonN( gCBP, VWPolygon2DObj( *this ), false , 32 ) );
	poly2DObj.GetPolygon( poly2DPolygonized );
	poly2DObj.DeleteObject( false );

	// normalize the vector
	VWPoint2D			vDir		= vDirection;
	vDir.Normalize();

	VWPoint2D			vPerpDir	= vDir.Perp();
	vPerpDir.Normalize();

	// four zero vectors
	VWPoint2D			vIMax, vIMin, vJMax, vJMin;

	const VWPoint2D&	vInit		= poly2DPolygonized.GetVertexAt( 0 );
	size_t				vertCount	= poly2DPolygonized.GetVertexCount();

	for ( size_t i = 1 ; i < vertCount ; i ++ ) {

		// get current
		const VWPoint2D&	vCurr	= poly2DPolygonized.GetVertexAt( i ) - vInit;

		// project the current onto directional vector and on its perpendicular
		VWPoint2D			vPara	= vDir     * ( vCurr.DotProduct( vDir     ) / vDir.DotProduct( vDir         ) );
		VWPoint2D			vPerp	= vPerpDir * ( vCurr.DotProduct( vPerpDir ) / vPerpDir.DotProduct( vPerpDir ) );

		// get normalized vectors
		VWPoint2D			vParaNorm	= vPara;
		VWPoint2D			vPerpNorm	= vPerp;
		vParaNorm.Normalize();
		vPerpNorm.Normalize();
		
		if ( ( vParaNorm - vDir ).Magnitude() < VWPoint2D::sEpsilon ) {
			if ( vPara.Magnitude() > vIMax.Magnitude() )
				vIMax	= vPara;
		}
		else {
			if ( vPara.Magnitude() > vIMin.Magnitude() )
				vIMin	= vPara;
		}

		if ( ( vPerpNorm - vPerpDir ).Magnitude() < VWPoint2D::sEpsilon ) {
			if ( vPerp.Magnitude() > vJMax.Magnitude() )
				vJMax	= vPerp;
		}
		else {
			if ( vPerp.Magnitude() > vJMin.Magnitude() )
				vJMin	= vPerp;
		}
	}

	// find the bounding box polygon in the given direction
	vUL	=	vInit + vIMin + vJMax;
	vUR	=	vInit + vIMax + vJMax;
	vLL	=	vInit + vIMin + vJMin;
	vLR	=	vInit + vIMax + vJMin;
}

/////////////////////////////////////////////////////////////////////////////////
// Mathematics
/////////////////////////////////////////////////////////////////////////////////

// join this polygon to the given polygon only if both polygons are not closed and have touching endpoints (within epsilon).
bool VWPolygon2D::AddAdjoiningPolygon(const VWPolygon2D& adjoiningPoly, double epsilon, bool checkTypes/*= false*/)
{
	// init.
	bool						doAdd			= true;
	bool						isAdded			= false;

	if ( this->IsClosed() || adjoiningPoly.IsClosed() )		// bad input! cannot concatenate closed polygons.
	{
		doAdd									= false;
	}

	if ( doAdd && ! isAdded )	// still not added.
	{
		if ( this->IsEmpty() )
		{
			(*this)								= adjoiningPoly;
			isAdded								= true;
		}
		else if ( adjoiningPoly.IsEmpty() )
		{
			isAdded								= true;
		}

		if ( ! isAdded )		// still not added.
		{
			size_t	thisPolyCnt	= this->GetVertexCount();
			size_t	adjPolyCnt	= adjoiningPoly.GetVertexCount();

			// validate.
			VWFC_ASSERT( ( thisPolyCnt > 0 ) && ( adjPolyCnt > 0 ) );			// should be already ensured (see IsEmpty check above).

			// get endpoints.
			const VWPoint2D&	thisBegPt		= this->GetVertexAt(	0 );
			const VWPoint2D&	thisEndPt		= this->GetVertexAt(	thisPolyCnt - 1 );
			const VWPoint2D&	adjBegPt		= adjoiningPoly.GetVertexAt(	0 );
			const VWPoint2D&	adjEndPt		= adjoiningPoly.GetVertexAt(	adjPolyCnt - 1 );

			// get endpoints types.
			EVertexType			thisBegType		= this->GetVertexType(	0 );
			EVertexType			thisEndType		= this->GetVertexType(	thisPolyCnt - 1 );
			EVertexType			adjBegType		= adjoiningPoly.GetVertexType(	0 );
			EVertexType			adjEndType		= adjoiningPoly.GetVertexType(	adjPolyCnt - 1 );

			// check if endpoints touch.
			doAdd								= false;
			bool				dontCheckTypes	= ( ! checkTypes );
			VWPolygon2D			firstToAdd, secondToAdd;
			if		( ( dontCheckTypes || ( thisEndType == adjBegType ) ) &&	// types check (conditional).
					  MathUtils::Equalish( thisEndPt, adjBegPt, epsilon ) )
			{
				firstToAdd						= (*this);
				secondToAdd						= adjoiningPoly;

				doAdd							= true;
			}
			else if	( ( dontCheckTypes || ( thisEndType == adjEndType ) ) &&	// types check (conditional).
					  MathUtils::Equalish( thisEndPt, adjEndPt, epsilon ) )
			{
				firstToAdd						= (*this);
				secondToAdd						= adjoiningPoly;
				secondToAdd.ReversePoints( checkTypes );

				doAdd							= true;
			}
			else if	( ( dontCheckTypes || ( thisBegType == adjEndType ) ) &&	// types check (conditional).
					  MathUtils::Equalish( thisBegPt, adjEndPt, epsilon ) )
			{
				firstToAdd						= adjoiningPoly;
				secondToAdd						= (*this);

				doAdd							= true;
			}
			else if	( ( dontCheckTypes || ( thisBegType == adjBegType ) ) &&	// types check (conditional).
					  MathUtils::Equalish( thisBegPt, adjBegPt, epsilon ) )
			{
				firstToAdd						= (*this);
				firstToAdd.ReversePoints( checkTypes );
				secondToAdd						= adjoiningPoly;

				doAdd							= true;
			}

			if ( doAdd )	// adding is possible (there are touching endpoints).
			{
				// add second poly to first poly.
				size_t				secPolyCnt	= secondToAdd.farrVertices.size();
				for ( size_t i = 1 ; i < secPolyCnt ; ++ i )	// start from second vertex (don't add first vertex)!
				{
					const SVertex&	vertex		= secondToAdd.farrVertices[ i ];
					firstToAdd.farrVertices.push_back( vertex );
				}

				// update this poly with the result.
				(*this)							= firstToAdd;

				isAdded							= true;
			}
		}
	}

	// recalculate bounding box if needed.
	if ( isAdded && ( ! fLockedForEdit ) )
	{
		this->RecalculateBoundingBox();
	}

	// result.
	return	( isAdded );
}

// find index of next 'apposite' polygon point (not too close).
bool VWPolygon2D::GetNextAppositePtIndex(size_t inPtIndex, size_t& outNextPtIndex, double dEpsilon) const
{
	bool				hasNext		= false;

	// check if polygon has at least 3 vertices.
	size_t				len			= this->GetVertexCount();
	if ( len < 3 )
	{
		return	false;
	}
	// validation.
	VWFC_ASSERT( inPtIndex < len );

	// the point.
	const VWPoint2D&	pt			= this->GetVertexAt( inPtIndex );

	size_t				counter		= inPtIndex;	// init.
	// loop all next points.
	for ( size_t i = 0 ; i < ( len - 1 ) ; ++ i )
	{
		++ counter;		// increase.

		// check if index is acceptable.
		if ( ( ! this->IsClosed() ) && ( counter >= len ) )
		{
			break;
		}

		// get index of next point.
		size_t			currIndex	= counter;
		if ( counter >= len )
		{
			currIndex				= ( counter % len );
		}
		
		const VWPoint2D&	currPt	= this->GetVertexAt( currIndex );

		if ( ! MathUtils::Equalish( pt, currPt, dEpsilon ) )	// not too close.
		{
			outNextPtIndex			= currIndex;
			hasNext					= true;
			break;
		}
	}

	// result.
	return	hasNext;
}

// find next 'apposite' polygon point (not too close).
bool VWPolygon2D::GetNextAppositePt(size_t inPtIndex, VWPoint2D& outNextPt, double dEpsilon) const
{
	size_t	nextPtIndex;
	bool	hasNext		= this->GetNextAppositePtIndex( inPtIndex, nextPtIndex, dEpsilon );
	if ( hasNext )
	{
		outNextPt		= this->GetVertexAt( nextPtIndex );
	}
	return	hasNext;
}

// find index of previous 'apposite' polygon point (not too close).
bool VWPolygon2D::GetPrevAppositePtIndex(size_t inPtIndex, size_t& outPrevPtIndex, double dEpsilon) const
{
	bool				hasPrev		= false;

	// check if polygon has at least 3 vertices.
	size_t				len			= this->GetVertexCount();
	if ( len < 3 )
	{
		return	false;
	}
	// validation.
	VWFC_ASSERT( inPtIndex < len );

	// the point.
	const VWPoint2D&	pt			= this->GetVertexAt( inPtIndex );

	int					counter		= inPtIndex;	// init.
	// loop all previous points.
	for ( size_t i = 0 ; i < ( len - 1 ) ; ++ i )
	{
		-- counter;		// increase.

		// check if index is acceptable.
		if ( ( ! this->IsClosed() ) && ( counter < 0 ) )
		{
			break;
		}

		// get index of previous point.
		size_t			currIndex	= counter;
		if ( counter < 0 )
		{
			currIndex				= ( len + counter );
			VWFC_ASSERT( currIndex < len );	// must not happen !
		}
		
		const VWPoint2D&	currPt	= this->GetVertexAt( currIndex );

		if ( ! MathUtils::Equalish( pt, currPt, dEpsilon ) )	// not too close.
		{
			outPrevPtIndex			= currIndex;
			hasPrev					= true;
			break;
		}
	}

	// result.
	return	hasPrev;
}

// find previous 'apposite' polygon point (not too close).
bool VWPolygon2D::GetPrevAppositePt(size_t inPtIndex, VWPoint2D& outPrevPt, double dEpsilon) const
{
	size_t	prevPtIndex;
	bool	hasPrev		= this->GetPrevAppositePtIndex( inPtIndex, prevPtIndex, dEpsilon );
	if ( hasPrev )
	{
		outPrevPt		= this->GetVertexAt( prevPtIndex );
	}
	return	hasPrev;
}

// find indices of the first 3 'apposite' polygon points (not too close and not collinear).
bool VWPolygon2D::GetAppositePtIndices(size_t& ptIndex0, size_t& ptIndex1, size_t& ptIndex2) const
{
	// init.
	ptIndex0 = ptIndex1 = ptIndex2	= 0;

	// check if polygon has at least 3 vertices.
	size_t				len			= this->GetVertexCount();
	if ( len < 3 )
	{
		return false;
	}

	// get first apposite vertex.
	ptIndex0						= 0;
	const VWPoint2D&	pt0			= this->GetVertexAt( ptIndex0 );

	bool				hasApposite	= false;
	// get second apposite vertex.
	ptIndex1						= 1;
	VWPoint2D			pt1;
	while ( ptIndex1 < len )
	{
		pt1							= this->GetVertexAt( ptIndex1 );
		if ( ! MathUtils::Equalish( pt0, pt1, VWPoint2D::sEpsilon ) )	// not too close.
		{
			hasApposite				= true;
			break;
		}
		++ ptIndex1;
	}

	// get third apposite vertex.
	if ( hasApposite )
	{
		hasApposite					= false;

		ptIndex2					= ( ptIndex1 + 1 );
		while ( ptIndex2 < len )
		{
			const VWPoint2D& pt2	= this->GetVertexAt( ptIndex2 );
			if ( ! VWPoint2D::ArePointsCollinear( pt0, pt1, pt2, VWPoint2D::sEpsilonCollinear ) )	// not collinear.
			{
				hasApposite			= true;
				break;
			}
			++ ptIndex2;
		}
	}
	
	// result.
	return	hasApposite;
}

// find the first 3 'apposite' polygon points (not too close and not collinear).
bool VWPolygon2D::GetThreeAppositePoints(VWPoint2D& pt0, VWPoint2D& pt1, VWPoint2D& pt2) const
{
	// init.
	size_t	ptIndex0, ptIndex1, ptIndex2;
	// find indices of the first 3 'apposite' polygon points (not too close and not collinear).
	bool	hasApposite	= this->GetAppositePtIndices( ptIndex0, ptIndex1, ptIndex2 );

	if ( hasApposite )
	{
		// get the corresponding polygon points.
		pt0				= this->GetVertexAt( ptIndex0 );
		pt1				= this->GetVertexAt( ptIndex1 );
		pt2				= this->GetVertexAt( ptIndex2 );
	}

	// result.
	return	hasApposite;
}

void VWPolygon2D::Transform(const VWTransformMatrix& mat)
{
	fdMinX		= 0;
	fdMaxX		= 0;
	fdMinY		= 0;
	fdMaxY		= 0;

	size_t	len	= farrVertices.size();
	for(size_t i=0; i<len; i++)
	{
		VWPoint2D		pt		= farrVertices[ i ].pt;
		VWPoint3D		p( pt.x, pt.y, 0 );
		p = mat.PointTransform( p );
		if ( i == 0 )
		{
			fdMinX		= p.x;
			fdMaxX		= p.x;
			fdMinY		= p.y;
			fdMaxY		= p.y;
		}
		else
		{
			fdMinX		= min( fdMinX, p.x );
			fdMaxX		= max( fdMaxX, p.x );
			fdMinY		= min( fdMinY, p.y );
			fdMaxY		= max( fdMaxY, p.y );
		}
		farrVertices[ i ].pt.SetPoint( p.x, p.y );
	}
}

void VWPolygon2D::MovePolygon(double cx, double cy)
{
	fdMinX		= 0;
	fdMaxX		= 0;
	fdMinY		= 0;
	fdMaxY		= 0;

	size_t	len	= farrVertices.size();
	for(size_t i=0; i<len; i++)
	{
		VWPoint2D &		pt	= farrVertices[i].pt;

		pt.x		+= cx;
		pt.y		+= cy;

		if ( i == 0 )
		{
			fdMinX		= pt.x;
			fdMaxX		= pt.x;
			fdMinY		= pt.y;
			fdMaxY		= pt.y;
		}
		else
		{
			fdMinX		= min( fdMinX, pt.x );
			fdMaxX		= max( fdMaxX, pt.x );
			fdMinY		= min( fdMinY, pt.y );
			fdMaxY		= max( fdMaxY, pt.y );
		}
	}
}

// rotate the polygon by the center of the bounding box
void VWPolygon2D::Rotate(double degAngle)
{
	double_gs	x		= fdMinX + ( fdMaxX - fdMinX ) / 2;
	double_gs	y		= fdMinY + ( fdMaxY - fdMinY ) / 2;
	this->MovePolygon( -x, -y );

	double_gs	alpha	= MathUtils::Deg2Rad( degAngle );
	double_gs	s		= sin( alpha );
	double_gs	c		= cos( alpha );
	TransformMatrix		rotationMatrix;
	rotationMatrix.SetToIdentity();
	rotationMatrix.SetRow0( NormalVector(  c, s, 0 ) );
	rotationMatrix.SetRow1( NormalVector( -s, c, 0 ) );
	rotationMatrix.SetRow2( NormalVector(  0, 0, 1 ) );
	this->Transform( rotationMatrix );

	this->MovePolygon( x, y );
}

// rotate by specified center
void VWPolygon2D::Rotate(const VWPoint2D& pt, double degAngle)
{
	this->MovePolygon( -pt.x, -pt.y );

	double_gs	alpha	= MathUtils::Deg2Rad( degAngle );
	double_gs	s		= sin( alpha );
	double_gs	c		= cos( alpha );
	TransformMatrix		rotationMatrix;
	rotationMatrix.SetToIdentity();
	rotationMatrix.SetRow0( NormalVector(  c, s, 0 ) );
	rotationMatrix.SetRow1( NormalVector( -s, c, 0 ) );
	rotationMatrix.SetRow2( NormalVector(  0, 0, 1 ) );
	this->Transform( rotationMatrix );

	this->MovePolygon( pt.x, pt.y );
}

// calculate the area of the polygon
// A = 1/2 Sum(i:=0 to N-1)( x(i)*y(i+1) - x(i+1)*y(i) )
double VWPolygon2D::GetArea() const
{
	double	dArea			= 0;

	size_t	nVertexCount	= farrVertices.size();
	if ( nVertexCount == 0 )
		return dArea;

	VWPoint2D		a		= farrVertices[ 0 ].pt;
	VWPoint2D		b;
	VWPoint2D		c;

	for(size_t i=1; i<nVertexCount-1; i++) {

		b		= farrVertices[ i ].pt;
		c		= farrVertices[ i + 1 ].pt;

		// Get the area
		double_gs	triArea		= VWPoint2D::TriArea( a, b, c );

		// Calculate the area
		dArea		= dArea + triArea;
	}

	/*double dOldArea = dArea;
	 ***********************************
	 * other NOT working VERSION
	 **********************************

	dArea			= 0;
	for(size_t i=0; i<nVertexCount-1; i++) {
		const	VWPoint2D&	p0		= farrVertices[ i ].pt;
		const	VWPoint2D&	p1		= farrVertices[ i + 1 ].pt;

		dArea		+= p0.x*p1.y - p1.x*p0.y;
	}
	dArea		/= 2;

	VWFC_ASSERT( VWFC::Math::MathUtils::Equalish( dArea, dOldArea, 0.0001 ) );
	*/

	return dArea;
}

bool VWPolygon2D::IsClockWise() const
{
	double	area	= this->GetArea();
	return area <= 0;
}

// calculate the perimeter of the polygon
double VWPolygon2D::GetPerimeter() const
{
	double	dPerimeter			= 0;

	size_t	nVertexCount	= farrVertices.size();
	if ( nVertexCount == 0 )
		return dPerimeter;

	size_t	len	= nVertexCount;
	if ( ! fbIsClosed )
		len--;
	for(size_t i=0; i<len; i++) {
		VWPoint2D	a, b;
		a			= farrVertices[ i ].pt;
		b			= i+1 < nVertexCount ? farrVertices[ i+1 ].pt : farrVertices[ 0 ].pt;

		dPerimeter	+= a.DistanceTo( b );
	}

	return dPerimeter;
}

double VWPolygon2D::GetLength() const
{
	double	length		= 0;

	size_t	vertsCnt	= this->GetVertexCount();
	for(size_t i=0; i<vertsCnt;) {
		EVertexType			vertType	= this->GetVertexType( i );
		if ( vertType == eVertexType_Corner ) {
			if ( i > 0 ) {
				const VWPoint2D&	prevPt		= this->GetVertexAt( i - 1 );
				const VWPoint2D&	thisPt		= this->GetVertexAt( i );

				length	+= prevPt.DistanceTo( thisPt );
			}
			i++;
		}
		else if ( vertType == eVertexType_Bezier ) {
			DSTOP((kVStanev, "not implemented"));
			length	= 0;
			break;
		}
		else if ( vertType == eVertexType_Cubic ) {
			DSTOP((kVStanev, "not implemented"));
			length	= 0;
			break;
		}
		else if ( vertType == eVertexType_Arc ) {
			const VWPoint2D&	prevPt		= this->GetVertexAt( (i > 0) ? (i - 1) : (farrVertices.size() - 1) );
			const VWPoint2D&	thisPt		= this->GetVertexAt( i );
			const VWPoint2D&	nextPt		= this->GetVertexAt( i + 1, true );
			VWArc2D		arc( prevPt, thisPt, nextPt );
			length		+= arc.GetLength();
			i			+= 2;
		}
		else if ( vertType == eVertexType_Radius ) {
			DSTOP((kVStanev, "not implemented"));
			length	= 0;
			break;
		}
	}

	return length;
}
// calculate the length of the curve formed by the polyline up to the specified point
// if the point is not on the poly 
double VWPolygon2D::GetLength(const VWPoint2D& pt, double dEpsilon) const
{
	double	length		= 0;

	size_t	vertsCnt	= this->GetVertexCount();
	for(size_t i=0; i<vertsCnt;) {
		EVertexType			vertType	= this->GetVertexType( i );
		if ( vertType == eVertexType_Corner ) {
			if ( i > 0 ) {
				const VWPoint2D&	prevPt		= this->GetVertexAt( i - 1 );
				const VWPoint2D&	thisPt		= this->GetVertexAt( i );

				if ( VWLine2D::PtOnLine( pt, prevPt, thisPt, dEpsilon ) ) {
					length	+= prevPt.DistanceTo( pt );
					// Finished, Stop!
					break;
				}
				else {
					length	+= prevPt.DistanceTo( thisPt );
				}
			}
			i++;
		}
		else if ( vertType == eVertexType_Bezier ) {
			DSTOP((kVStanev, "not implemented"));
			length	= 0;
			break;
		}
		else if ( vertType == eVertexType_Cubic ) {
			DSTOP((kVStanev, "not implemented"));
			length	= 0;
			break;
		}
		else if ( vertType == eVertexType_Arc ) {
			const VWPoint2D&	prevPt		= this->GetVertexAt( (i > 0) ? (i - 1) : (farrVertices.size() - 1) );
			const VWPoint2D&	thisPt		= this->GetVertexAt( i );
			const VWPoint2D&	nextPt		= this->GetVertexAt( i + 1, true );
			VWArc2D		arc( prevPt, thisPt, nextPt );
			if ( arc.IsPtOnArc( pt, dEpsilon ) ) {
				length	+= arc.GetLength( pt );
				// Finished, Stop!
				break;
			}
			else {
				length		+= arc.GetLength();
				i			+= 2;
			}
		}
		else if ( vertType == eVertexType_Radius ) {
			DSTOP((kVStanev, "not implemented"));
			length	= 0;
			break;
		}
	}

	return length;
}

double VWPolygon2D::GetLength(size_t fromVert, size_t toVert) const
{
	double	length		= 0;

	size_t	vertsCnt	= this->GetVertexCount();
	for(size_t i=fromVert; i<toVert && i<vertsCnt;) {
		EVertexType			vertType	= this->GetVertexType( (i + 1) % vertsCnt );
		if ( vertType == eVertexType_Corner ) {
			const VWPoint2D&	prevPt		= this->GetVertexAt( i );
			const VWPoint2D&	thisPt		= this->GetVertexAt( i + 1, true );

			length	+= prevPt.DistanceTo( thisPt );
			i++;
		}
		else if ( vertType == eVertexType_Bezier ) {
			DSTOP((kVStanev, "not implemented"));
			length	= 0;
			break;
		}
		else if ( vertType == eVertexType_Cubic ) {
			DSTOP((kVStanev, "not implemented"));
			length	= 0;
			break;
		}
		else if ( vertType == eVertexType_Arc ) {
			const VWPoint2D&	thisPt		= this->GetVertexAt( i );
			const VWPoint2D&	nextPt		= this->GetVertexAt( i + 1, true );
			const VWPoint2D&	nextNextPt	= this->GetVertexAt( i + 2, true );
			VWArc2D		arc( thisPt, nextPt, nextNextPt );
			length		+= arc.GetLength();
			i			+= 2;
		}
		else if ( vertType == eVertexType_Radius ) {
			DSTOP((kVStanev, "not implemented"));
			length	= 0;
			break;
		}
	}

	return length;
}

// returns a point along the poly on the specified offset
bool VWPolygon2D::GetPtOnPoly(double offset, VWPoint2D& outPt) const
{
	// polygonize.
	VWPolygon2D	polygonized;
	this->GetPolygonized( polygonized );

	// use polygonization to find the point on the poly.

	bool		result		= false;
	double		length		= 0;
	size_t		vertsCnt	= polygonized.GetVertexCount();

	for ( size_t i = 1 ; i < vertsCnt ; ++ i )
	{
		const VWPoint2D&	prevPt		= polygonized.GetVertexAt( i - 1 );
		const VWPoint2D&	thisPt		= polygonized.GetVertexAt( i );

		double				newLen		= length + prevPt.DistanceTo( thisPt );

		if ( MathUtils::GEish( newLen, offset, VWPoint2D::sEpsilon ) )
		{
			double		dist			= offset - length;
			VWPoint2D	n				= ( thisPt - prevPt ).Normalize();
			outPt						= prevPt + ( dist * n );
			result						= true;
			// Finished, Stop!
			break;
		}
		else
		{
			length			= newLen;
		}
	}

	return	result;
}

bool VWPolygon2D::GetPtOnPoly(double offset, VWPoint2D& outPt, VWPoint2D& outNormal) const
{
	bool		result		= false;
	double		length		= 0;
	size_t		vertsCnt	= this->GetVertexCount();
	for(size_t i=0; i<vertsCnt;) {
		EVertexType			vertType	= this->GetVertexType( i );
		if ( vertType == eVertexType_Corner ) {
			if ( i > 0 ) {
				size_t				prevIndex	= i - 1;
				const VWPoint2D&	prevPt		= this->GetVertexAt( prevIndex );
				const VWPoint2D&	thisPt		= this->GetVertexAt( i );

				double	newLen	= length + prevPt.DistanceTo( thisPt );
				if ( MathUtils::GEish( newLen, offset, VWPoint2D::sEpsilon ) ) {
					double		dist	= offset - length;
					VWPoint2D	n		= (thisPt - prevPt).Normalize();
					outPt				= prevPt + dist * n;
					result				= true;

					// calc the normal
					bool	atStart	= MathUtils::Equalish( dist, 0, VWPoint2D::sEpsilon );
					bool	atEnd	= MathUtils::Equalish( newLen, offset, VWPoint2D::sEpsilon );
					if ( atStart ||	atEnd )
					{

						VWPoint2D	v1, v2;
						double		ang;
						if ( atStart )
						{
							const VWPoint2D&	prevPrevPt	= this->GetVertexAt( (prevIndex > 0) ? (prevIndex - 1) : (vertsCnt - 1) );
							v1								= (prevPt - prevPrevPt).Normalize();
							v2								= (thisPt - prevPt).Normalize();
							ang								= Abs( (prevPrevPt - prevPt).CalcAng180() - (thisPt - prevPt).CalcAng180() );
						}
						else
						{
							const VWPoint2D&	nextPt	= this->GetVertexAt( (prevIndex > 0) ? (prevIndex - 1) : (vertsCnt - 1) );
							v1							= (thisPt - prevPt).Normalize();
							v2							= (nextPt - thisPt).Normalize();
							ang							= Abs( (prevPt - thisPt).CalcAng180() - (nextPt - thisPt).CalcAng180() );
						}

						if ( ! this->IsClosed() && atStart && prevIndex == 0 )
						{
							outNormal		= - v2.Perp();
						}
						else if ( ! this->IsClosed() && atEnd && prevIndex == vertsCnt - 2 )
						{
							outNormal		= - v1.Perp();
						}
						else
						{
							outNormal		= - ((v1.Perp() + v2.Perp()) / 2).Normalize();
							double		div	= sin( MathUtils::Deg2Rad( ang / 2 ) );
							if ( ! MathUtils::Equalish( div, 0, VWPoint2D::sEpsilon ) )
							{
								// calculate the common length
								outNormal		/= div;
							}
						}
					}
					else
					{
						outNormal			= - (thisPt - prevPt).Normalize().Perp();
					}
					
					// Finished, Stop!
					break;
				}
				else {
					length	= newLen;
				}
			}
			i++;
		}
		else if ( vertType == eVertexType_Bezier ) {
			DSTOP((kVStanev, "not implemented"));
			length	= 0;
			break;
		}
		else if ( vertType == eVertexType_Cubic ) {
			DSTOP((kVStanev, "not implemented"));
			length	= 0;
			break;
		}
		else if ( vertType == eVertexType_Arc ) {
			DSTOP((kVStanev, "not implemented"));
			length	= 0;
			break;
		}
		else if ( vertType == eVertexType_Radius ) {
			DSTOP((kVStanev, "not implemented"));
			length	= 0;
			break;
		}
	}

	return result;
}

// finds the closest point on poly of the given point.
VWPoint2D VWPolygon2D::GetClosestPtOnPoly(const VWPoint2D& pt, double& outDist, int& outOnSegIndex) const
{
	VWPoint2D						closestPt;

	size_t							vertCnt		= this->GetVertexCount();
	// validate.
	VWFC_ASSERT( vertCnt > 0 );	// poly is empty! incorrect.

	if ( vertCnt < 2 )	// single point poly.
	{
		closestPt								= this->GetVertexAt( 0 );
		outDist									= pt.DistanceTo( closestPt );
		outOnSegIndex							= 0;
	}
	else				// poly has at least two points.
	{
		// compute poly segments to process.
		bool						isClosed	= this->IsClosed();
		size_t						procCnt		= vertCnt;
		if ( ! isClosed )
		{
			-- procCnt;
		}

		for ( size_t i = 0 ; i < procCnt ; ++ i )
		{
			double					currDist;	// for min distance to this poly segment.
			VWPoint2D				currPt;		// for closest point on this poly segment.

			bool					isOnSeg		= true;

			// this index.
			size_t					thisIndex	= i;

			// index of next poly point.
			size_t					nextIndex;
			if ( isClosed )			nextIndex	= ( thisIndex + 1 ) % procCnt;
			else					nextIndex	= ( thisIndex + 1 );

			// get vertex type.
			EVertexType				vertType	= this->GetVertexType( nextIndex );

			if ( vertType == eVertexType_Corner )	// this segment is a line.
			{
				const VWLine2D&		polyLine	= this->GetLineAt( thisIndex );
		
				// get the closest point on line.
				const VWPoint2D&	projPt		= polyLine.PtPerpLine( pt );
				if ( polyLine.PtOnLine( projPt, VWPoint2D::sEpsilon ) )	// projection is on line.
				{
					currDist					= pt.DistanceTo( projPt );
					currPt						= projPt;
				}
				else
				{
					isOnSeg						= false;
				}
			}
			else if ( vertType == eVertexType_Bezier )
			{
				DSTOP( ( kVStanev, "not implemented" ) );
				isOnSeg							= false;
			}
			else if ( vertType == eVertexType_Cubic )
			{
				DSTOP( ( kVStanev, "not implemented" ) );
				isOnSeg							= false;
			}
			else if ( vertType == eVertexType_Arc )	// the segment is an arc.
			{
				// get next of next index.
				size_t				nextNextInx	= size_t(-1);
				if ( isClosed )
				{
					nextNextInx					= ( nextIndex + 1 ) % procCnt;
				}
				else if ( nextIndex < procCnt )
				{
					nextNextInx					= ( nextIndex + 1 );
				}

				if ( nextNextInx != size_t(-1) )
				{
					// get poly arc.
					const VWPoint2D&	prevPt	= this->GetVertexAt( thisIndex );
					const VWPoint2D&	thisPt	= this->GetVertexAt( nextIndex );
					const VWPoint2D&	nextPt	= this->GetVertexAt( nextNextInx );
					VWArc2D				polyArc( prevPt, thisPt, nextPt );

					// get the closest point on arc.
					const VWPoint2D&	projPt	= polyArc.PtPerpArc( pt );
					if ( polyArc.IsPtOnArc( projPt, VWPoint2D::sEpsilon ) )
					{
						currDist				= pt.DistanceTo( projPt );
						currPt					= projPt;
					}
					else
					{
						isOnSeg					= false;
					}

					++	i;										// increase counter!
					nextIndex					= nextNextInx;	// update next index.
				}
				else
				{
					isOnSeg						= false;
				}
			}
			else if ( vertType == eVertexType_Radius ) {
				DSTOP((kVStanev, "not implemented"));
				break;
			}

			// if projection is not on segment.
			if ( ! isOnSeg )
			{
				// get segment ends.
				const VWPoint2D&	segPt0	= this->GetVertexAt( thisIndex );
				const VWPoint2D&	segPt1	= this->GetVertexAt( nextIndex );

				double			distToPt0	= pt.DistanceTo( segPt0 );
				double			distToPt1	= pt.DistanceTo( segPt1 );
				if ( distToPt0 < distToPt1 )
				{
					currDist				= distToPt0;
					currPt					= segPt0;
				}
				else
				{
					currDist				= distToPt1;
					currPt					= segPt1;
				}
			}

			if ( ( thisIndex == 0 ) || ( currDist < outDist ) )
			{
				// the following lines will be reached at least once (see that vertsCnt is > 1)
				outDist						= currDist;
				closestPt					= currPt;
				outOnSegIndex				= thisIndex;

				if ( MathUtils::Equalish( outDist, 0.0, VWPoint2D::sEpsilon ) )	// stop if point lies on the poly segment.
				{
					break;
				}
			}
		}
	}

	return	closestPt;
}

// finds the closest projection point on the polygon of the given point (if exists). Note: works on polygon with only corner vertices.
// returns closest projection point, squared distance to the projection point and index of the segment where the projection point lies.
bool VWPolygon2D::GetClosestProjPtOnPoly_Fast(const VWPoint2D& pt, double& outSqrDist, int& outOnSegIndex, VWPoint2D& outProjPt) const
{
	size_t							vertCnt		= this->GetVertexCount();
	// validate.
	VWFC_ASSERT( vertCnt > 0 );		// poly is empty! bad input.

	bool							isFound		= false;
	if ( vertCnt > 1 )				// poly has at least two points.
	{
		// compute poly segments to process.
		bool						isClosed	= this->IsClosed();
		size_t						procCnt		= vertCnt;
		if ( ! isClosed )
		{
			-- procCnt;
		}

		for ( size_t i = 0 ; i < procCnt ; ++ i )
		{
			const VWLine2D&			polyLine	= this->GetLineAt( i );
			const VWPoint2D&		lnPt0		= polyLine.GetStart();
			const VWPoint2D&		lnPt1		= polyLine.GetEnd();

			// get the projection point on line.
			const VWPoint2D&		projPt		= polyLine.PtPerpLine( pt );

			// use a fast check here whether the projection is on the line.
			// get line bounding box.
			VWRectangle2D	bboxLine(	lnPt0.x, lnPt1.x,	// left / right.
										lnPt0.y, lnPt1.y );	// top / bottom.
			// check if point is inside the line bounding box: this will mean that the point lies on the line segment for sure.
			if ( bboxLine.IsPtInside( projPt ) )
			{
				// find current square distance to the projection.
				double				curSqrDist	= VWPoint2D::DistanceSqare( pt.x, pt.y, projPt.x, projPt.y );;

				if ( ( ! isFound ) || ( curSqrDist < outSqrDist ) )
				{
					isFound							= true;

					// the following lines will be reached at least once (see that vertsCnt is > 1)
					outSqrDist						= curSqrDist;
					outProjPt						= projPt;
					outOnSegIndex					= i;

					if ( MathUtils::Equalish( outSqrDist, 0.0, VWPoint2D::sEpsilon ) )	// stop if point lies on the poly segment.
					{
						break;
					}
				}
			}
		}
	}

	// result.
	return	isFound;
}

// calculates an offset ring poly ouside or inside of this poly.
void VWPolygon2D::GetOffsetRingPoly(double offset, bool doOutsideOffset, VWPolygon2D& offsetRingPoly) const
{
	offsetRingPoly.ClearVertices();

	// get poly center.
	VWPoint2D				cenPt	= this->GetCenter();

	// get the sign of the usable directions.
	double					sign	=  1.0;
	if ( this->IsClockWise() )
	{
		sign						= -1.0;
	}

	if ( ! doOutsideOffset )		// doing inside offset.
	{
		sign						*= -1.0;
	}

	VWPoint2D				prevPt;
	size_t					cnt		= this->GetVertexCount();
	for ( size_t i = 0 ; i < cnt ; ++ i )
	{
		const	VWPoint2D&	pt		= this->GetVertexAt( i );			// current poly point.

		if ( i == 0 )
		{
			prevPt					= this->GetVertexAt( cnt - 1 );		// get last point.
		}
		VWPoint2D			nextPt	= this->GetVertexAt( ( i + 1 ) % cnt );

		VWPoint2D			vecPrev	= ( pt - prevPt ).Normalize();		// prev normal vector.
		VWPoint2D			vecNext	= ( nextPt - pt ).Normalize();		// next normal vector.

		// get usable direction.
		VWPoint2D			dir		= ( vecPrev + vecNext ).Normalize();
		dir							= dir.Perp();
		dir							*= sign;

		// if the direction isn't usable use the poly center to find a usable.
		if ( ! MathUtils::Equalish( dir.Magnitude(), 1.0, VWPoint2D::sEpsilon ) )
		{
			dir						= ( pt - cenPt ).Normalize();
		}

		// calculate and store the offset point.
		VWPoint2D			ringPt	= pt + ( offset * dir );
		offsetRingPoly.AddVertex( ringPt );

		prevPt						= pt;								// store prev point.
	}

	// set closed.
	offsetRingPoly.SetClosed( this->IsClosed() );
}

// check if polygon is equalish to this
bool VWPolygon2D::EqualishTo(const VWPolygon2D& poly) const
{
	size_t	thisCnt	= this->GetVertexCount();
	size_t	polyCnt	= poly.GetVertexCount();

	bool	bEqual	= true;
	if ( thisCnt == polyCnt ) {

		// find positions coincidence
		bool	bPosCoincide	= false;
		size_t	thisSupl, polySupl;
		for ( size_t i = 0 ; i < thisCnt ; i ++ ) {
			for ( size_t j = 0 ; j < thisCnt ; j ++ ) {

				const VWPoint2D&	pt1	= this->GetVertexAt( i );
				const VWPoint2D&	pt2	= poly.GetVertexAt( j );
				if ( MathUtils::Equalish( pt1, pt2, VWPoint2D::sEpsilon ) ) {
					thisSupl		= i;
					polySupl		= j;
					bPosCoincide	= true;
					break;
				}
			}
			if ( bPosCoincide )
				break;
		}
		
		if ( bPosCoincide && thisCnt > 0 ) {
			for ( size_t i = 0 ; i < thisCnt ; i ++ ) {

				// corresponding indices
				size_t	thisIndex	= ( i + thisSupl ) % thisCnt;
				size_t	polyIndex	= ( i + polySupl ) % thisCnt;

				// check types
				EVertexType	vType1	= this->GetVertexType( thisIndex );
				EVertexType	vType2	= poly.GetVertexType( polyIndex );
				if ( vType1 != vType2 ) {
					bEqual	= false;
					break;
				}
				// check radiuses
				if ( vType1 == eVertexType_Arc && 
					this->GetArcVertexRadius( thisIndex ) != poly.GetArcVertexRadius( polyIndex ) ) {
					bEqual	= false;
					break;
				}
				// check positions
				if ( ! MathUtils::Equalish( this->GetVertexAt( thisIndex ), poly.GetVertexAt( polyIndex ), VWPoint2D::sEpsilon ) ) {
					bEqual	= false;
					break;
				}
			}
		}
		else {
			bEqual	= false;
		}
	}
	else {
		bEqual	= false;
	}
	return bEqual;
}

// is specified point on the edge of the specified polygon
bool VWPolygon2D::IsPtOnPoly2D(const VWPoint2D& pt, double dEpsilon) const
{
	bool	onPoly	= false;
	bool	inPoly	= false;
	this->IsPtOnPoly2D( pt.x, pt.y, onPoly, inPoly, dEpsilon );
	return onPoly;
}

bool VWPolygon2D::IsPtOnPoly2D_Fast(const VWPoint2D& inPt, int& outOnSegIndex, double epsilon) const
{
	// init.
	bool						isOnPoly	= false;

	size_t						polyPtCnt	= farrVertices.size();
	if ( polyPtCnt > 1 )		// poly has at least two points.
	{
		// compute poly edges to process.
		size_t					procPtCnt	= polyPtCnt;
		if ( ! this->IsClosed() )
		{
			-- procPtCnt;
		}

		// loop through all edges of the polygon.
		for ( size_t i = 0 ; i < procPtCnt ; ++ i )		// edge from V[i] to V[i+1].
		{
			// first edge point.
			const VWPoint2D&	pt			= farrVertices[ i ].pt;

			// second edge point.
			size_t				nextIndex	= ( i < ( polyPtCnt - 1 ) ) ? ( i + 1 ) : 0;
			const VWPoint2D&	nextPt		= farrVertices[ nextIndex ].pt;

			// use the fast check here.
			if ( VWLine2D::PtOnLine_Fast( inPt, pt, nextPt, epsilon ) )
			{
				isOnPoly					= true;
				outOnSegIndex				= i;
				break;
			}
		}
	}

	// result.
	return	isOnPoly;
}

// determines if a point is in the area of a given polygon (the 2D projection)
bool VWPolygon2D::IsPtInPoly2D(double ptX, double ptY, double dEpsilon) const
{
	bool	onPoly	= false;
	bool	inPoly	= false;
	this->IsPtOnPoly2D( ptX, ptY, onPoly, inPoly, dEpsilon );
	return inPoly;
}

bool VWPolygon2D::IsPtInPoly2D(const VWPoint2D& pt, double dEpsilon) const
{
	bool	onPoly	= false;
	bool	inPoly	= false;
	this->IsPtOnPoly2D( pt.x, pt.y, onPoly, inPoly, dEpsilon );
	return inPoly;
}

bool VWPolygon2D::IsPtInPoly2D_Fast(double ptX, double ptY, double dEpsilon) const
{
	// not using epsilon -- but we should
	dEpsilon;

	int		cn			= 0;    // the crossing number counter

	size_t	polyPtCnt	= farrVertices.size();
	// loop through all edges of the polygon
	for (size_t i=0; i<polyPtCnt; i++) {    // edge from V[i] to V[i+1]
		const VWPoint2D&	pt		= farrVertices[i].pt;
		const VWPoint2D&	nextPt	= i+1 < polyPtCnt ? farrVertices[i+1].pt : farrVertices[0].pt;

		double				Vix		= pt.x;
		double				Viy		= pt.y;
		double				Vi1x	= nextPt.x;
		double				Vi1y	= nextPt.y;

		if (((Viy <= ptY) && (Vi1y > ptY))    // an upward crossing
			|| ((Viy > ptY) && (Vi1y <= ptY))) { // a downward crossing
			// compute the actual edge-ray intersect x-coordinate
			double vt = (double)(ptY - Viy) / (Vi1y - Viy);
			if ( ptX < Vix + vt * (Vi1x - Vix) ) // P.x < intersect
				++cn;   // a valid crossing of y=P.y right of P.x
		}
	}

	return (cn&1);    // 0 if even (out), and 1 if odd (in)
}

bool VWPolygon2D::IsPtOnPoly2D(double ptX, double ptY, double dEpsilon) const
{
	bool	onPoly	= false;
	bool	inPoly	= false;
	this->IsPtOnPoly2D( ptX, ptY, onPoly, inPoly, dEpsilon );
	return onPoly;
}

// combined operations
bool VWPolygon2D::IsPtInOrOnPoly2D(double ptX, double ptY, double dEpsilon) const
{
	bool	onPoly	= false;
	bool	inPoly	= false;
	this->IsPtOnPoly2D( ptX, ptY, onPoly, inPoly, dEpsilon );
	return onPoly || inPoly;
}

bool VWPolygon2D::IsPtInOrOnPoly2D(const VWPoint2D& pt, double dEpsilon) const
{
	bool	onPoly	= false;
	bool	inPoly	= false;
	this->IsPtOnPoly2D( pt.x, pt.y, onPoly, inPoly, dEpsilon );
	return onPoly || inPoly;
}

void VWPolygon2D::IsPtOnPoly2D(const VWPoint2D& pt, bool& outOnPoly, bool& outInPoly, double dEpsilon) const
{
	this->IsPtOnPoly2D( pt.x, pt.y, outOnPoly, outInPoly, dEpsilon );
}

void VWPolygon2D::IsPtOnPoly2D(double ptX, double ptY, bool& outOnPoly, bool& outInPoly, double dEpsilon) const
{
	VWPoint2D	inPt( ptX, ptY );

	outOnPoly			= false;
	int		cn			= 0;    // the crossing number counter

	size_t	polyPtCnt	= farrVertices.size();
	// loop through all edges of the polygon
	for (size_t i=0; i<polyPtCnt; i++) {    // edge from V[i] to V[i+1]
		const VWPoint2D&	pt		= farrVertices[i].pt;
		const VWPoint2D&	nextPt	= i+1 < polyPtCnt ? farrVertices[i+1].pt : farrVertices[0].pt;

		outOnPoly	= VWLine2D::PtOnLine( inPt, pt, nextPt, dEpsilon );
		// if it is on poly break -- we have 'on poly' and dont have 'in poly'
		if ( outOnPoly ) 
			break;

		double				Vix		= pt.x;
		double				Viy		= pt.y;
		double				Vi1x	= nextPt.x;
		double				Vi1y	= nextPt.y;

		if (((Viy <= ptY) && (Vi1y > ptY))    // an upward crossing
			|| ((Viy > ptY) && (Vi1y <= ptY))) { // a downward crossing
			// compute the actual edge-ray intersect x-coordinate
			double vt = (double)(ptY - Viy) / (Vi1y - Viy);
			if ( ptX < Vix + vt * (Vi1x - Vix) ) // P.x < intersect
				++cn;   // a valid crossing of y=P.y right of P.x
		}
	}

	// if we have 'on poly' means that we dont have 'in poly'
	if ( ! outOnPoly ) {
		outInPoly	= ( cn & 1 ) ? true : false;    // 0 if even (out), and 1 if odd (in)
	}
	else {
		outInPoly	= false;
	}

/*	size_t	len		= farrVertices.size();
	if ( len <= 2 )
		return false;

	// find Inside poly 
	VWPoint2D	p	= pt;
	VWPoint2D	thisPt	= farrVertices[ len - 1 ].pt;
	VWPoint2D	lastPt;

	bool right1		= thisPt.x >= pt.x;
	bool high1		= thisPt.y >= pt.y;

	bool right2, high2;

	short	cross	= 0;
	for (size_t i=0; i<len; i++)
	{
		lastPt	= thisPt;
		thisPt	= farrVertices[ i ].pt;

		if ( thisPt.Equal( lastPt, dEpsilon ) )
			continue;

		VWLine2D	line( thisPt, lastPt );
		if ( line.PtOnLine( p, dEpsilon ) )
		{
			cross	= 0;
			break;
		}

		right2 = thisPt.x >= pt.x;
		high2  = thisPt.y >= pt.y;

		if ( high1 != high2 )
		{
			if ( right1 != right2 )
			{
				// cross=maybe			
				double_gs	test = lastPt.x + (thisPt.x - lastPt.x)*(p.y - lastPt.y)/(double_gs)(thisPt.y - lastPt.y);

				if (test == 0)
				{
					cross = 0;
					break;
				//	return false;
				}
				else if (test >= p.x) 
					cross++;
			}
			else {
				if (right1) {
					if ( thisPt.x == pt.x && lastPt.x == pt.x ) {
						//AND right2 follows, since theY are Equalrich:test this for new Clip,
						// if pt lay on horizontal edge, then it is not
						// Inside
						
						cross	= 0;
						break;
						//return false;
					}
					cross++;
				}
			}
		}
		right1	= right2;
		high1	= high2;
	}


	bool res = (cross & 1) != 0L;
	return res;*/
}

// return true if the specified line intersects the poly
bool VWPolygon2D::IsIntersecting(const VWLine2D& line, double dEpsilon, VWPoint2D* /*out*/ pInsertPt/*=NULL*/) const
{
	// not using epsilon -- but we should
	dEpsilon;

	size_t		vertsCnt	= farrVertices.size();
	size_t		len			= vertsCnt;
	if ( len == 0 )
		return false;

	// dont check the last legment if this is open poly
	if ( ! fbIsClosed )
		len --;

	if ( len == 0 )
		return false;

	VWPoint2D	insertPt;
	VWPoint2D	p			= farrVertices[ 0 ].pt;
	for(size_t i=1; i<=len; i++)
	{
		VWPoint2D		q;
		if ( i == vertsCnt )	q	= farrVertices[ 0 ].pt;
		else					q	= farrVertices[ i ].pt;

		VWLine2D	curLine( p, q );
		if ( line.IntersectLineSegments( curLine, insertPt ) )
		{
			if ( pInsertPt )
				(*pInsertPt)		= insertPt;
			return true;
		}

		p		= q;
	}

	return false;
}

bool VWPolygon2D::IsIntersecting(const VWPoint2D& a, const VWPoint2D& b, double dEpsilon, VWPoint2D* /*out*/ pInsertPt/*=NULL*/) const
{
	// not using epsilon -- but we should
	dEpsilon;

	size_t		len			= farrVertices.size();
	if ( len == 0 )
		return false;

	// dont check the last legment if this is open poly
	if ( ! fbIsClosed )
		len --;

	if ( len == 0 )
		return false;

	VWPoint2D	insertPt;
	VWPoint2D	p			= farrVertices[ 0 ].pt;
	for(size_t i=1; i<=len; i++)
	{
		VWPoint2D		q;
		if ( i == len )	q	= farrVertices[ 0 ].pt;
		else			q	= farrVertices[ i ].pt;

		VWLine2D	curLine( p, q );
		if ( curLine.IntersectLineSegments( a, b, insertPt ) )
		{
			if ( pInsertPt )
				(*pInsertPt)		= insertPt;
			return true;
		}

		p		= q;
	}

	return false;
}

// return true if the specified line intersects the poly
bool VWPolygon2D::IsIntersecting(const VWLine2D& line, double dEpsilon, TSTLPoints2DArray& outArrIntersectPts) const
{
	bool bSimilarPts = true;
	size_t		vertsCnt	= farrVertices.size();
	size_t		len			= vertsCnt;
	if ( len == 0 )
		return false;

	// dont check the last legment if this is open poly
	if ( ! fbIsClosed )
		len --;

	if ( len == 0 )
		return false;

	VWPoint2D	insertPt;
	VWPoint2D	p			= farrVertices[ 0 ].pt;
	bool foundIntersectPoint = false;
	for(size_t i=1; i<=len; i++)
	{
		VWPoint2D		q;
		if ( i == vertsCnt )	q	= farrVertices[ 0 ].pt;
		else					q	= farrVertices[ i ].pt;

		VWLine2D	curLine( p, q );
		if ( line.IntersectLineSegments( curLine, insertPt ) )
		{
			size_t arraySize = outArrIntersectPts.size();

			if( arraySize == 0 ) {
				outArrIntersectPts.push_back( insertPt );
				foundIntersectPoint = true;
			}
			else {
				for( size_t k = 0; k < arraySize; k++ ) {
					if( !WorldPtsAreRelativelyClose( insertPt, outArrIntersectPts[k], dEpsilon ) ) {
						bSimilarPts = false;
					}
				}

				if( !bSimilarPts ) {
					outArrIntersectPts.push_back( insertPt );
					foundIntersectPoint = true;
				}
			}
		}

		p		= q;
	}

	return foundIntersectPoint;
}

bool VWPolygon2D::IsIntersecting(const VWPoint2D& a, const VWPoint2D& b, double dEpsilon, TSTLPoints2DArray& outArrIntersectPts) const
{
	VWLine2D tempLine( a, b );

	return this->IsIntersecting( tempLine, dEpsilon, outArrIntersectPts );

	/*
	// not using epsilon -- but we should
	dEpsilon;

	size_t		len			= farrVertices.size();
	if ( len == 0 )
		return false;

	// dont check the last legment if this is open poly
	if ( ! fbIsClosed )
		len --;

	if ( len == 0 )
		return false;

	VWPoint2D	insertPt;
	VWPoint2D	p			= farrVertices[ 0 ].pt;
	bool foundIntersectPoint = false;
	for(size_t i=1; i<=len; i++)
	{
		VWPoint2D		q;
		if ( i == len )	q	= farrVertices[ 0 ].pt;
		else			q	= farrVertices[ i ].pt;

		VWLine2D	curLine( p, q );
		if ( curLine.IntersectLineSegments( a, b, insertPt ) )
		{
			outArrIntersectPts.push_back( insertPt );
			foundIntersectPoint = true;
		}

		p		= q;
	}

	return foundIntersectPoint;*/
}

// return true if the given polygon and this polygon intersects each other
// intersect 'true' means that they have intersection points or same points
bool VWPolygon2D::IsIntersecting(const VWPolygon2D& poly, double dEpsilon) const
{
	size_t		polyLen		= poly.farrVertices.size();
	if ( polyLen == 0 )
		return false;

	// dont consider the last line (end-start) if poly is open
	if ( ! poly.IsClosed() )
		polyLen--;

	bool		bInters		= false;
	for(size_t i=0; i<polyLen; i++)
	{
		VWLine2D		line	= poly.GetLineAt( i );
		if ( this->IsIntersecting( line, dEpsilon, NULL ) ) {
			bInters		= true;
			break;
		}
	}

	return bInters;
}

bool VWPolygon2D::IsIntersecting(const VWPolygon2D& poly, bool& outIsTouching, double dEpsilon) const
{
	VWPoint2D	pt;
	return this->IsIntersecting( poly, outIsTouching, pt, dEpsilon );
}

// retun true if the given polygon and this polygon intersects each other (including touching)
// outIsTouching returns if they are tourching
bool VWPolygon2D::IsIntersecting(const VWPolygon2D& poly, bool& outIsTouching, VWPoint2D& outIntersPt, double dEpsilon) const
{
	outIsTouching	= false;

	size_t		polyLen		= poly.farrVertices.size();
	if ( polyLen == 0 )
		return false;

	// dont consider the last line (end-start) if poly is open
	if ( ! poly.IsClosed() )
		polyLen--;

	bool		bInters		= false;
	for(size_t i=0; i<polyLen && ! bInters; i++)
	{
		VWLine2D		line	= poly.GetLineAt( i );
	
		size_t		vertsCnt	= farrVertices.size();
		size_t		len			= vertsCnt;
		if ( len > 0 ) {
			// dont check the last legment if this is open poly
			if ( ! fbIsClosed )
				len --;

			if ( len > 0 ) {
				VWPoint2D	insertPt;
				VWPoint2D	p			= farrVertices[ 0 ].pt;
				for(size_t i=1; i<=len; i++)
				{
					VWPoint2D		q;
					if ( i == vertsCnt )	q	= farrVertices[ 0 ].pt;
					else					q	= farrVertices[ i ].pt;

					VWLine2D	curLine( p, q );
					bool		onSegment;
					VWPoint2D	intersPt;
					if ( line.IntersectLines( curLine, intersPt, onSegment ) ) {
						if ( onSegment ) {
							bInters			= true;
							outIntersPt		= intersPt;
							if ( ! line.PtOnLinePoints( outIntersPt, dEpsilon ) &&
								 ! curLine.PtOnLinePoints( outIntersPt, dEpsilon ) )
							{
								outIsTouching	= false;
								break;
							}
							else {
								outIsTouching	= true;
							}
						}
					}
					else if ( onSegment ) {
						bInters			= true;
						outIntersPt		= intersPt;
						outIsTouching	= true;
					}

					p		= q;
				}
			}
		}
	}

	return bInters;
}

// return true if part of a polygon is inside a vicinity of this polygon.
bool VWPolygon2D::IsInVicinity(const VWPolygon2D& poly, double dVicinity) const
{
	bool		inVicinity			= false;

	// check closeness of the given poly's points to this polygon.
	size_t		polyLen				= poly.GetVertexCount();
	for ( size_t i = 0 ; i < polyLen ; ++ i ) {
		const VWPoint2D&	pt		= poly.GetVertexAt( i );

		if ( this->IsPtOnPoly2D( pt, dVicinity ) ) {
			inVicinity				= true;
			break;
		}
	}

	// check closeness of this polygon's points to the given poly.
	if ( ! inVicinity ) {

		polyLen						= this->GetVertexCount();
		for ( size_t i = 0 ; i < polyLen ; ++ i ) {
			const VWPoint2D&	pt	= this->GetVertexAt( i );

			if ( poly.IsPtOnPoly2D( pt, dVicinity ) ) {
				inVicinity			= true;
				break;
			}
		}
	}

	// check for intersections.
	if ( ( ! inVicinity ) && this->IsIntersecting( poly, dVicinity ) ) {	// if intersecting then it is inside vicinity.
		inVicinity					= true;
	}

	return	inVicinity;
}

// returns true if the given polygon is fully within this polygon
bool VWPolygon2D::IsFullyInside(const VWPolygon2D& poly, double dEpsilon) const
{
	bool	bIsFullyInside	= false;
	bool	bInters			= this->IsIntersecting( poly, dEpsilon );
	if ( bInters ) {
		bIsFullyInside	= false;
	}
	else {
		bIsFullyInside			= true;

		size_t		polyLen		= poly.farrVertices.size();
		for(size_t i=0; i<polyLen; i++)
		{
			VWPoint2D		pt		= poly.farrVertices[ i ].pt;
			bool			onPoly	= false;
			bool			inPoly	= false;
			this->IsPtOnPoly2D( pt, onPoly, inPoly, dEpsilon );

			if ( ! onPoly && ! inPoly ) {
				bIsFullyInside	= false;
				break;	// STOP!
			}
		}
	}

	return bIsFullyInside;
}

// returns true if the given polygon is fully outside this polygon
bool VWPolygon2D::IsFullyOutside(const VWPolygon2D& poly, double dEpsilon) const
{
	bool	bIsFullyOutside	= false;
	bool	bInters			= this->IsIntersecting( poly, dEpsilon );
	if ( bInters ) {
		bIsFullyOutside	= false;
	}
	else {
		bIsFullyOutside			= true;

		size_t		polyLen		= poly.farrVertices.size();
		for(size_t i=0; i<polyLen; i++) {
			VWPoint2D		pt		= poly.farrVertices[ i ].pt;
			bool			onPoly	= false;
			bool			inPoly	= false;
			this->IsPtOnPoly2D( pt, onPoly, inPoly, dEpsilon );

			if ( onPoly || inPoly ) {
				bIsFullyOutside	= false;
				break;	// STOP!
			}
		}
	}

	return bIsFullyOutside;
}


// returns true if the given polygon is fully within this polygon, including having common points
bool VWPolygon2D::IsInside(const VWPolygon2D& poly, double dEpsilon) const
{
	bool	bIsInside			= true;

	size_t		polyLen		= poly.farrVertices.size();
	for(size_t i=0; i<polyLen; i++)
	{
		VWPoint2D		pt		= poly.farrVertices[ i ].pt;
		bool			onPoly	= false;
		bool			inPoly	= false;
		this->IsPtOnPoly2D( pt, onPoly, inPoly, dEpsilon );

		if ( ! onPoly && ! inPoly ) {
			bIsInside	= false;
			break;	// STOP!
		}
	}

	return bIsInside;
}

// returns true if the given polygon is fully outside this polygon, including having common points
bool VWPolygon2D::IsOutside(const VWPolygon2D& poly, double dEpsilon) const
{
	bool bIsOutside			= true;

	size_t		polyLen		= poly.farrVertices.size();
	for(size_t i=0; i<polyLen; i++) {
		VWPoint2D		pt		= poly.farrVertices[ i ].pt;
		bool			onPoly	= false;
		bool			inPoly	= false;
		this->IsPtOnPoly2D( pt, onPoly, inPoly, dEpsilon );

		if ( onPoly || inPoly ) {
			bIsOutside	= false;
			break;	// STOP!
		}
	}

	return bIsOutside;
}

// get geometry center of the polygon
VWPoint2D VWPolygon2D::GetCenter() const
{
	VWPoint2D	centerPt;

	size_t		polyLen		= farrVertices.size();
	for(size_t i=0; i<polyLen; i++) {
		const VWPoint2D&	pt		= farrVertices[ i ].pt;
		if ( i == 0 )
			centerPt		= pt;
		else
			centerPt		+= pt;
	}

	centerPt	/= polyLen;

	return centerPt;
}

// get an inside point: prefer points that are not too close to the polygon boundary.
bool VWPolygon2D::GetPtInsidePoly(VWPoint2D& insidePt, double dEpsilon) const
{
	bool			isInside	= false;

	// get first 3 'apposite' polygon points - not too close and not collinear.
	VWPoint2D		pt0, pt1, pt2;
	if ( this->GetThreeAppositePoints( pt0, pt1, pt2 ) )
	{
		isInside				= true;

		// get the median direction.
		VWPoint2D	midDir		= ( ( pt0 - pt1 ) + ( pt2 - pt1 ) ).Normalize();
		// correct direction.
		if ( this->IsPtInPoly2D( ( pt1 - ( midDir * VWPoint2D::sEpsilon ) ), 0.0 ) )
		{
			midDir				*= - 1.0;
		}
		else if ( ! this->IsPtInPoly2D( ( pt1 + ( midDir * VWPoint2D::sEpsilon ) ), 0.0 ) )
		{
			isInside			= false;
			VWFC_ASSERT( false );	// a degenerate polygon!
		}

		if ( isInside )	// we have a correct direction.
		{
			// start from the distance: from pt1 to center of the triangle defined by the three points.
			double	dist		= pt1.DistanceTo( ( pt0 + pt2 + pt1 ) / 3 );

			do
			{
				insidePt		= ( pt1 + ( midDir * dist ) );
				isInside		= this->IsPtInPoly2D( insidePt, dEpsilon );

				// decrease the distance.
				dist			*= 0.5;
			}
			while ( ( ! isInside ) && ( dist > dEpsilon ) );
		}
	}

	// result.
	return	isInside;
}

// get random point that lies in the poly.
// XXX Bobi: not advisable to use:
// for polygon with small area it could take very long time to finish !
VWPoint2D VWPolygon2D::GetRandomPtInPoly(double dEpsilon) const
{
	VWPoint2D	pt;
	if ( MathUtils::Greaterish( Abs( this->GetArea() ), 0.0, ( this->GetPerimeter() * dEpsilon ) ) )
	{
		do
		{
			pt.x	= MathUtils::RealRandom( fdMinX, fdMaxX );
			pt.y	= MathUtils::RealRandom( fdMinY, fdMaxY );
		}
		while ( ! this->IsPtInPoly2D( pt, dEpsilon ) );
	}
	else
	{
		// this is a degenerate polygon (very small area).
		// return the center point in this case.
		pt			= this->GetCenter();
		VWFC_ASSERT( false );		// degenerate polygon.
	}

	return pt;
}

// get bounding points along line
// the result points are projection on the line of the two points from the poly wich are bound points along the line
void VWPolygon2D::GetBoundingProjOnLine(VWPoint2D&/*in-out*/ minProjPt, VWPoint2D&/*in-out*/ maxProjPt, double dEpsilon) const
{
	// not using epsilon -- but we should
	dEpsilon;

	VWLine2D			line( minProjPt, maxProjPt );
	
	// min and max distance from the begining of the line (face bearing line)
	double				minProjDist;
	double				maxProjDist;
	size_t				vertCnt	= farrVertices.size();
	for(size_t v=0; v<vertCnt; v++)
	{
		VWPoint2D	pt			= farrVertices[ v ].pt;
		VWPoint2D	projPt		= line.PtPerpLine( pt );
		//double		dist		= ( projPt - pt ).Magnitude();
		double		projDist	= ( projPt - line.GetStart() ).Magnitude();
		if ( v == 0 )
		{
			// first time - init
			minProjPt = maxProjPt			= projPt;
			minProjDist = maxProjDist		= projDist;
		}
		else
		{
			if ( projDist < minProjDist )
			{
				minProjPt		= projPt;
				minProjDist		= projDist;
			}
			if ( projDist > maxProjDist )
			{
				maxProjPt		= projPt;
				maxProjDist		= projDist;
			}
		}
	}
}

// get an inside point near the vertex at
VWPoint2D VWPolygon2D::GetInsidePtNearVertexAt(size_t at)
{
	VWFC_ASSERT( at < farrVertices.size() );

	const double	kNearPtEpsilon	= 1000 * VWPoint2D::sEpsilon;

	VWPoint2D	pt		= this->GetVertexAt( at );
	VWPoint2D	prevPt;
	if ( at < 1 )	prevPt	= this->GetVertexAt( this->GetVertexCount() - 1 );
	else			prevPt	= this->GetVertexAt( at - 1 );
	VWPoint2D		nextPt	= this->GetVertexAt( at + 1, true );

	VWLine2D	line( prevPt, nextPt );
	VWPoint2D	midDir	= line.CenterPt() - pt;
	if ( midDir.Magnitude() < VWPoint2D::sEpsilon ) {
		VWPoint2D	vec		= nextPt - pt;
		midDir.SetPoint( vec.y, -vec.x );
	}
	midDir.Normalize();

	VWPoint2D	checkPt	= pt + midDir * kNearPtEpsilon;
	if ( this->IsPtInPoly2D( checkPt, VWPoint2D::sEpsilon ) ) {
		return	checkPt;
	}
	else {
		checkPt	= pt - midDir * kNearPtEpsilon;
		VWFC_ASSERT( this->IsPtInPoly2D( checkPt, VWPoint2D::sEpsilon ) );
		return	checkPt;
	}
}

// get set of inside points near polygon's vertices
TSTLPoints2DArray VWPolygon2D::GetInsideNearPts()
{
	size_t	ptsCnt	= this->GetVertexCount();
	TSTLPoints2DArray	resultArray;
	for ( size_t i = 0 ; i < ptsCnt ; i ++ ) {
		resultArray.push_back( this->GetInsidePtNearVertexAt( i ) );
	}
	return	resultArray;
}

void VWPolygon2D::PointsTransform(const VWTransformMatrix& matrix)
{
 	size_t	len	= farrVertices.size();
	for(size_t i=0; i<len; i++) {
		VWPoint2D&	pt		= farrVertices[i].pt;
		pt					= matrix.PointTransform( pt );
	}

	if ( ! fLockedForEdit )
	{
		this->RecalculateBoundingBox();
	}
}

void VWPolygon2D::InversePointsTransform(const VWTransformMatrix& matrix)
{
 	size_t	len	= farrVertices.size();
	for(size_t i=0; i<len; i++) {
		VWPoint2D&	pt		= farrVertices[i].pt;
		pt					= matrix.InversePointTransform( pt );
	}

	if ( ! fLockedForEdit )
	{
		this->RecalculateBoundingBox();
	}
}

// is pt (x0,y0) is inside or/and on the specified triangle (x1,y1):(x2,y2):(x3,y3)
void VWPolygon2D::IsPtInTriangle( 	double x0, double y0,
									double x1, double y1,
									double x2, double y2,
									double x3, double y3,
									bool& outInTri, bool& outOnTri)
{
	VWPolygon2D::IsPtInTriangle( x0, y0, x1, y1, x2, y2, x3, y3, outInTri, outOnTri, VWPoint2D::sEpsilon );
}

void VWPolygon2D::IsPtInTriangle(	double x0, double y0,
									double x1, double y1,
									double x2, double y2,
									double x3, double y3,
									bool& outInTri, bool& outOnTri,
									double epsilon)
{
	// init.
	outInTri		=	false;
	outOnTri		=	false;
	
	double		b0	=  (x2 - x1) * (y3 - y1) - (x3 - x1) * (y2 - y1);
	
#define EQ_ZERO(val)	MathUtils::Equalish( val, 0.0, epsilon )
#define GR_ZERO(val)	MathUtils::Greaterish( val, 0.0, epsilon )

	if ( ! EQ_ZERO(b0) ) {	// triangle points are not collinear - this is a "good" triangle.
	
		double		b1	= ((x2 - x0) * (y3 - y0) - (x3 - x0) * (y2 - y0)) / b0;
		double		b2	= ((x3 - x0) * (y1 - y0) - (x1 - x0) * (y3 - y0)) / b0;
		double		b3	= ((x1 - x0) * (y2 - y0) - (x2 - x0) * (y1 - y0)) / b0;

//		outInTri		= b1 > 0 && b2 > 0 && b3 > 0;
//		outOnTri		= (b1 == 0 && b2 > 0 && b3 > 0)		||
//						  (b1 > 0 && b2 == 0 && b3 > 0)		||
//						  (b1 > 0 && b2 > 0 && b3 == 0)		||
//						  // is on points
//						  (b1 > 0 && b2 == 0 && b3 == 0)	||
//						  (b1 == 0 && b2 > 0 && b3 == 0)	||
//						  (b1 == 0 && b2 == 0 && b3 > 0) ;

		outInTri		= GR_ZERO(b1) && GR_ZERO(b2) && GR_ZERO(b3);
		outOnTri		= ( EQ_ZERO(b1) && GR_ZERO(b2) && GR_ZERO(b3) )	||
						  ( GR_ZERO(b1) && EQ_ZERO(b2) && GR_ZERO(b3) )	||
						  ( GR_ZERO(b1) && GR_ZERO(b2) && EQ_ZERO(b3))	||
						  // is on points
						  ( GR_ZERO(b1) && EQ_ZERO(b2) && EQ_ZERO(b3) )	||
						  ( EQ_ZERO(b1) && GR_ZERO(b2) && EQ_ZERO(b3) )	||
						  ( EQ_ZERO(b1) && EQ_ZERO(b2) && GR_ZERO(b3) ) ;
	}
	else {	// if the three triangle points are close to collinear.
		
		// check if point is on the triangle's border.
		int	onSegIndex	=	-1;
		int	onPtIndex	=	-1;
		VWPolygon2D::IsPtOnTriangleBorder( x0, y0, x1, y1, x2, y2, x3, y3,
										   onSegIndex, onPtIndex, epsilon );
		outOnTri		=	( onSegIndex != -1 ) || ( onPtIndex != -1 );
	}

#undef EQ_ZERO
#undef GR_ZERO
}

// is pt (x0,y0) is inside or/and on the specified triangle (x1,y1):(x2,y2):(x3,y3)
// outOnSegIndex returns:
//	if (x0,y0) on [(x1,y1),(x2,y2] -> 0
//	if (x1,y1) on [(x2,y2),(x3,y3] -> 1
//	if (x2,y2) on [(x3,y3),(x1,y1] -> 2
// outOnPtIndex returns:
//	if (x0,y0) == (x1,y1) -> 0
//	if (x0,y0) == (x2,y2) -> 1
//	if (x0,y0) == (x3,y3) -> 2
void VWPolygon2D::IsPtInTriangle(	double x0, double y0,
									double x1, double y1,
									double x2, double y2,
									double x3, double y3,
									bool& outInTri,
									int& outOnSegIndex,
									int& outOnPtIndex,
									double epsilon)
{
	// init.
	outInTri		=	false;
	outOnSegIndex	=	-1;
	outOnPtIndex	=	-1;
	
#define	EQ_ZERO(val)		MathUtils::Equalish( val, 0.0, epsilon )
#define	GR_ZERO(val)		MathUtils::Greaterish( val, 0.0, epsilon )

	//									|x1 y1 1|
	// compute the det(p1, p2, p3)	=	|x2 y2 1|
	//									|x3 y3 1|
	double		b0	=  (x2 - x1) * (y3 - y1) - (x3 - x1) * (y2 - y1);
	
	// check if the three (triangle) points are close to collinear.
	if ( ! EQ_ZERO(b0) ) {	// triangle points are not collinear - this is a "good" triangle.
	
		// get det(p0, p2, p3), det(p0, p1, p3) and det(p0, p1, p2).
		double		b1	= ((x2 - x0) * (y3 - y0) - (x3 - x0) * (y2 - y0));
		double		b2	= ((x3 - x0) * (y1 - y0) - (x1 - x0) * (y3 - y0));
		double		b3	= ((x1 - x0) * (y2 - y0) - (x2 - x0) * (y1 - y0));
		
		// calculate/use barycentric coordinates of p0.
		b1	/=	b0;
		b2	/=	b0;
		b3	/=	b0;
	
		// according to the already found barycentric coordinates:
		//	outInTri		= b1 > 0 && b2 > 0 && b3 > 0;
		//	outOnTri		= (b1 == 0 && b2 > 0 && b3 > 0)		||
		//					  (b1 > 0 && b2 == 0 && b3 > 0)		||
		//					  (b1 > 0 && b2 > 0 && b3 == 0)		||
		//					  // is on points
		//					  (b1 > 0 && b2 == 0 && b3 == 0)	||
		//					  (b1 == 0 && b2 > 0 && b3 == 0)	||
		//					  (b1 == 0 && b2 == 0 && b3 > 0) ;

		outInTri		= GR_ZERO(b1) && GR_ZERO(b2) && GR_ZERO(b3);

		// is on segment
		if ( ( EQ_ZERO(b1) && GR_ZERO(b2) && GR_ZERO(b3) ) ) {
			outOnSegIndex	= 1;
		}
		else if ( ( GR_ZERO(b1) && EQ_ZERO(b2) && GR_ZERO(b3) ) ) {
			outOnSegIndex	= 2;
		}
		else if ( ( GR_ZERO(b1) && GR_ZERO(b2) && EQ_ZERO(b3)) ) {
			outOnSegIndex	= 0;
		}
		
		// is on points
		if ( ( GR_ZERO(b1) && EQ_ZERO(b2) && EQ_ZERO(b3) ) ) {
			outOnPtIndex	= 0;
		}
		else if ( ( EQ_ZERO(b1) && GR_ZERO(b2) && EQ_ZERO(b3) )	) {
			outOnPtIndex	= 1;
		}
		else if ( ( EQ_ZERO(b1) && EQ_ZERO(b2) && GR_ZERO(b3) ) ) {
			outOnPtIndex	= 2;
		}
		
	}
	else {	// if the three triangle points are close to collinear.
	
		// check if point is on the triangle's border.
		VWPolygon2D::IsPtOnTriangleBorder( x0, y0, x1, y1, x2, y2, x3, y3,
										   outOnSegIndex, outOnPtIndex, epsilon );
	}

#undef EQ_ZERO
#undef GR_ZERO
}

// is pt (x0,y0) on the border of the specified triangle (x1,y1):(x2,y2):(x3,y3)
// works in all cases (including triangle with 3 vertices close to collinear).
// outOnSegIndex returns:
//	if (x0,y0) on [(x1,y1),(x2,y2)] -> 0
//	if (x1,y1) on [(x2,y2),(x3,y3)] -> 1
//	if (x2,y2) on [(x3,y3),(x1,y1)] -> 2
// outOnPtIndex returns:
//	if (x0,y0) == (x1,y1) -> 0
//	if (x0,y0) == (x2,y2) -> 1
//	if (x0,y0) == (x3,y3) -> 2
void VWPolygon2D::IsPtOnTriangleBorder(	double x0, double y0,
										double x1, double y1,
										double x2, double y2,
										double x3, double y3,
										int& outOnSegIndex,
										int& outOnPtIndex,
										double epsilon)
{
	outOnSegIndex	= -1;
	outOnPtIndex	= -1;

#ifndef	EQ_ZERO
#define	EQ_ZERO(val)		MathUtils::Equalish( val, 0.0, epsilon )
#endif
#define	BETWN(val, a, b)	MathUtils::IsBetween( val, a, b, epsilon )
	
	// -------------------------------------------------------------------------------------
	//							|x0 y0 1|
	// get det(p0, p2, p3)	=	|x2 y2 1|, det(p0, p1, p3) and det(p0, p1, p2) (analogously).
	//							|x3 y3 1|
	// -------------------------------------------------------------------------------------
	double		b1	= ((x2 - x0) * (y3 - y0) - (x3 - x0) * (y2 - y0));
	double		b2	= ((x3 - x0) * (y1 - y0) - (x1 - x0) * (y3 - y0));
	double		b3	= ((x1 - x0) * (y2 - y0) - (x2 - x0) * (y1 - y0));
	
	
	// if the three triangle points are close to collinear.
	
	if ( BETWN(x0, x1, x2) && BETWN(y0, y1, y2) && EQ_ZERO(b3) ) {	// pt on triangle's segment 0.
	
		if 		( EQ_ZERO(x0 - x1) && EQ_ZERO(y0 - y1) ) {				// coincides with p1.
			outOnPtIndex	=	0;
		}
		else if ( EQ_ZERO(x0 - x2) && EQ_ZERO(y0 - y2) ) {				// coincides with p2.
			outOnPtIndex	=	1;
		}
		else {
			outOnSegIndex	=	0;
		}
	}
	else if ( BETWN(x0, x2, x3) && BETWN(y0, y2, y3) && EQ_ZERO(b1) ) {	// pt on triangle's segment 1.
	
		if 		( EQ_ZERO(x0 - x2) && EQ_ZERO(y0 - y2) ) {					// coincides with p2.
			outOnPtIndex	=	1;
		}
		else if ( EQ_ZERO(x0 - x3) && EQ_ZERO(y0 - y3) ) {					// coincides with p3.
			outOnPtIndex	=	2;
		}
		else {
			outOnSegIndex	=	1;
		}
	}
	else if ( BETWN(x0, x1, x3) && BETWN(y0, y1, y3) && EQ_ZERO(b2) ) {	// pt on triangle's segment 2.
	
		if 		( EQ_ZERO(x0 - x1) && EQ_ZERO(y0 - y1) ) {					// coincides with p1.
			outOnPtIndex	=	0;
		}
		else if ( EQ_ZERO(x0 - x3) && EQ_ZERO(y0 - y3) ) {					// coincides with p3.
			outOnPtIndex	=	2;
		}
		else {
			outOnSegIndex	=	2;
		}
	}

#ifdef	EQ_ZERO
#undef	EQ_ZERO
#endif
#undef BETWN
}

// faster check if two polygons intersect each other. NOTE: no epsilon is used.
// before checking for actual intersections, this function checks if bounding boxes have common area.
// NOTE: if returned 'outNoCommonArea' is true, then polygons don't have common area for sure, otherwise ...
// ... if 'outNoCommonArea' is false then polygons probably have common area, but this still has to be checked.
bool VWPolygon2D::ArePolygonsIntersecting_Fast(const VWPolygon2D& polyA, const VWPolygon2D& polyB, bool& outNoCommonArea)
{
	// XXX Bobi:
	// use a very small epsilon (1e-16) when checking if two lines are intersecting (see the call of AreLineSegmentsIntersecting below).
	// this epsilon will catch the case when a lines are "touching" each other:
	// i.e. at least one of the end points of a line lie exactly on the other line or match its end points.
	const double		kIntersSegsEpsilon	= 0.000000000000001;	// 1e-16.

	// init.
	bool				areIntersecting		= false;
	outNoCommonArea							= true;

	// validate.
	if ( polyA.IsEmpty() || polyB.IsEmpty() )
	{
		ASSERTN( kVStanev, false );	// bad input !
		return	false;
	}

	const VWRectangle2D&		bboxA		= polyA.GetPolygonBounds();
	const VWRectangle2D&		bboxB		= polyB.GetPolygonBounds();

	// first: check if bounding boxes of the two polygons do not have common area, in such case they do not overlap.
	if ( ! bboxA.IsOutside( bboxB ) )							// bounding boxes have common area.
	{
		size_t					vertCntA	= polyA.GetVertexCount();
		size_t					cntA		= ( polyA.IsClosed() ) ? vertCntA : ( vertCntA - 1 );

		size_t					vertCntB	= polyB.GetVertexCount();
		size_t					cntB		= ( polyB.IsClosed() ) ? vertCntB : ( vertCntB - 1 );

		// next: check for intersection between the two polygons.
		for ( size_t i = 0 ; i < cntA ; ++ i )
		{
			const VWPoint2D&		v1		= polyA.GetVertexAt( i );

			size_t					iNext	= ( i + 1 );
			iNext							= ( iNext < vertCntA ) ? iNext : 0;
			const VWPoint2D&		v2		= polyA.GetVertexAt( iNext );

			VWRectangle2D			bboxLineA( v1.x, v2.x, v1.y, v2.y );		// get line bounding box.

			// check if line's bounding box has common area with the bounding box of the other poly, in such case they do not overlap.
			if ( ! bboxB.IsOutside( bboxLineA ) )				// they have common area.
			{
				outNoCommonArea					= false;						// we can't say that they don't have common area any more.
																				// from now on we are not sure about it.

				for ( size_t j = 0 ; j < cntB ; ++ j )
				{
					const VWPoint2D&	p1		= polyB.GetVertexAt( j );

					size_t				jNext	= ( j + 1 );
					jNext						= ( jNext < vertCntB ) ? jNext : 0;
					const VWPoint2D&	p2		= polyB.GetVertexAt( jNext );

					VWRectangle2D		bboxLineB( p1.x, p2.x, p1.y, p2.y );	// get line bounding box.

					// check if bounding boxes of the two lines have common area, in such case they do not overlap.
					if ( ! bboxLineA.IsOutside( bboxLineB ) &&	// the two lines have common area.
						 VWLine2D::AreLineSegmentsIntersecting( p1, p2, v1, v2, kIntersSegsEpsilon ) )	// use fast check here !
					{
						areIntersecting			= true;
						break;
					}
				}

				if ( areIntersecting )
				{
					break;
				}
			}
		}

		// additional check:
		// if polygons don't intersect and all bounding boxes of polyA's segments are outside the bounding box of the polyB, ...
		// ... then the polygons might still have common area if the bounding box of polyB is inside the bounding box of polyA.
		if ( outNoCommonArea && bboxA.IsInside( bboxB ) )
		{
			outNoCommonArea						= false;
		}
	}

	// result.
	return	areIntersecting;
}

// polygonization.
// ---------------------------------------------------------------------------------------------------------------------

// get polygonization:
// has parameters to control the density of the resulting polygon. recommended to use defaults !
// if 'interpolate' is true, then bezier and cubic segments will be interpolated by adding cubic vertices, ...
// ... so result will be interpolation poly with only corner and cubic vertices.
void VWPolygon2D::GetPolygonized(	VWPolygon2D& outPolygon,
									bool	interpolate		/*=false*/,
									short	splineRecusLevel/*=kPolygonize_MaxRecursLevel*/,
									double	arcDegRes		/*=kPolygonize_ArcResolution*/,
									size_t	iterationCount	/*=kPolygonize_MaxIterCount*/) const
{
	// init.
	outPolygon.ClearVertices();

	size_t				ptsCnt		= farrVertices.size();

	// validation.
	if ( ptsCnt <= 1 )
	{
		ASSERTN( kVStanev, false );
		return;
	}

	// init the array of extended vertices.
	TExtVertexArray		arrVertices;
	for ( size_t i = 0 ; i < ptsCnt ; ++ i )
	{
		SExtVertex		extVert		= farrVertices[ i ];
		arrVertices.push_back( extVert );
	}

	bool				isClosed	= this->IsClosed();

	// check for a special case: first point coincides with the last one.
	if (	( ptsCnt > 2 ) &&
			( MathUtils::Equalish( arrVertices[ 0 ].pt, arrVertices[ ptsCnt - 1 ].pt, VWPoint2D::sEpsilon ) ) )
	{
		// validate: coincident points must have same vertex types.
		ASSERTN( kVStanev, ( arrVertices[ 0 ].type == arrVertices[ ptsCnt - 1 ].type ) );

		// remove first point.
		arrVertices.erase( arrVertices.begin() );
		-- ptsCnt;

		isClosed					= true;
	}

	outPolygon.SetClosed( isClosed );

	// prepare cubic vertices.
	this->CalculatePolyline( arrVertices, iterationCount );
	this->SwapControlPts( arrVertices );

	size_t				lastIndex	= ( ptsCnt - 1 );
	SExtVertex			lastVert	= arrVertices[ lastIndex ];

	SExtVertex			thisVert	= arrVertices[ 0 ];
	SExtVertex			nextVert	= arrVertices[ 1 ];

	if ( isClosed && ( thisVert.type != eVertexType_Corner ) )	// the polyline is closed and start vertex is not corner.
	{
		// polygonize the start poly segment.
		this->PolygonizeSection( lastVert, thisVert, nextVert, outPolygon, interpolate, splineRecusLevel, arcDegRes );
	}
	else														// the start point of an open polyline must be a corner vertex.
	{
		lastVert.pt					= thisVert.pt;

		outPolygon.AddVertex( lastVert.pt );					// add as corner vertex.
	}

	// polygonize next segments.
	for ( size_t i = 2 ; i < ptsCnt ; ++ i )
	{
		lastVert					= thisVert;
		thisVert					= nextVert;
		nextVert					= arrVertices[ i ] ;

		// polygonize current poly segment.
		this->PolygonizeSection( lastVert, thisVert, nextVert, outPolygon, interpolate, splineRecusLevel, arcDegRes );
	}

	lastVert						= thisVert;
	thisVert						= nextVert;
	nextVert						= arrVertices[ 0 ];

	if ( isClosed && ( thisVert.type != eVertexType_Corner ) )	// the polyline is closed and last vertex is not corner.
	{
		// polygonize the last poly segment.
		this->PolygonizeSection( lastVert, thisVert, nextVert, outPolygon, interpolate, splineRecusLevel, arcDegRes );
	}
	else														// the start point of an open polyline must be a corner vertex.
	{
		outPolygon.AddVertex( thisVert.pt );					// add as corner vertex.
	}

	if ( isClosed && ( ! outPolygon.IsEmpty() ) )
	{
		outPolygon.AddVertex( outPolygon.GetVertexAt( 0 ) );	// enclose the polygon.
	}

	// clean up.
	outPolygon.CleanUpPolygon( VWPoint2D::sEpsilon );
}

void VWPolygon2D::CalculatePolyline(	TExtVertexArray& arrVertices,
										size_t iterationCount/*=kPolygonize_MaxIterCount*/) const
// This function seems to iterate through the polyline looking for
// cubic control points.  If it does not find any it exits.  If some are found they are processed.
//
// According to SteveJ:
// This is computing the controlPt member of the PolylineVert structure for the cubic verts.
// The controlPt is a cached value, which, one would think, could be updated at edit/reset time.
// We are always doing this in Render.
///////////////////////////////////////////////////////////////////////////////
{
	size_t			ptsCnt			= arrVertices.size();

	if ( ptsCnt > 1 )		// there are at least two points.
	{
		// check if there are cubic vertices and set the control pt to pt for them (the cubic vertices).
		bool		bFound			= false;
		for ( size_t k = 1 ; k <= ptsCnt ; ++ k )
		{
			SExtVertex*	vertex		= & arrVertices[ k - 1 ];
			if ( vertex->type == eVertexType_Cubic )
			{
				vertex->controlPt	= vertex->pt;
				bFound				= true;
			}
		}

		// don't bother to itterate if no cubics are found.
		if ( ! bFound )
		{
			return;
		}

		size_t		prevIndex		= ( ptsCnt - 1 );
		size_t		i				= ptsCnt;

		VWPoint2D	vPrev;
		VWPoint2D	vNext;

		// here we set 5 iterations.
		for ( size_t j = 1 ; j <= iterationCount ; ++ j )
		{
			for ( size_t nextIndex = 1 ; nextIndex <= ptsCnt ; ++ nextIndex )
			{
				SExtVertex*		currVertex		= & arrVertices[ i - 1 ];
				if ( currVertex->type == eVertexType_Cubic )
				{
					SExtVertex	prevVertex		= arrVertices[ prevIndex - 1 ];
					switch ( prevVertex.type )
					{
						case eVertexType_Cubic:
							{
								vPrev			= prevVertex.controlPt;
								break;
							}
						case eVertexType_Corner:
							{
								VWPoint2D	tempPt	= prevVertex.pt	- arrVertices[ i - 1 ].controlPt;
								vPrev				= tempPt + prevVertex.pt;
								break;
							}
						default:
							{
								vPrev			= prevVertex.pt;
								break;
							}
					}

					SExtVertex	nextVertex		= arrVertices[ nextIndex - 1 ];

					switch ( nextVertex.type )
					{
						case eVertexType_Cubic:
							{
								vNext			= nextVertex.controlPt;
								break;
							}
						case eVertexType_Corner:
							{
								VWPoint2D	tempPt	= nextVertex.pt - arrVertices[ i - 1 ].controlPt;
								vNext				= tempPt + nextVertex.pt;
								break;
							}
						default:
							{
								vNext			= nextVertex.pt;
								break;
							}
					}

					// with third order.
					double	divResultX			= ( currVertex->pt.x - ( vPrev.x + vNext.x ) * 0.5 ) / 3.0;
					// third order.
					double	divResultY			= ( currVertex->pt.y - ( vPrev.y + vNext.y ) * 0.5 ) / 3.0;

					// update the polyline.
					currVertex->controlPt.x		= ( currVertex->pt.x + divResultX );
					currVertex->controlPt.y		= ( currVertex->pt.y + divResultY );
				}

				prevIndex	= i;
				i			= nextIndex;

			} // end for nextIndex loop.
		} // end for j loop.
	}
}

void VWPolygon2D::SwapControlPts(TExtVertexArray& arrVertices) const
{
	size_t				ptsCnt	= arrVertices.size();
	for ( size_t i = 1 ; i <= ptsCnt ; ++ i )
	{
		SExtVertex*		vertex	= & arrVertices[ i - 1 ];
		// swap control pt and pt for cubic vertices.
		if ( vertex->type == eVertexType_Cubic )
		{
			VWPoint2D	tmp		= vertex->controlPt;
			vertex->controlPt	= vertex->pt;
			vertex->pt			= tmp;
		}
	}
}

void VWPolygon2D::PolygonizeSection(	const SExtVertex& lastVert, const SExtVertex& thisVert, const SExtVertex& nextVert,
										VWPolygon2D& outPolygon,
										bool	interpolate		/*=false*/,
										short	splineRecusLevel/*=kPolygonize_MaxRecursLevel*/,
										double	arcDegRes		/*=kPolygonize_ArcResolution*/) const
{
	switch ( thisVert.type )
	{
		case eVertexType_Corner:
			{
				outPolygon.AddVertex( thisVert.pt );		// add.
			}
			break;
		case eVertexType_Cubic:
		case eVertexType_Bezier:
			{
				VWPoint2D		p0, p2;

				if ( lastVert.type	== eVertexType_Corner )	p0	= lastVert.pt;
				else										p0	= ( thisVert.pt + lastVert.pt ) * 0.5;

				if ( nextVert.type == eVertexType_Corner )	p2	= nextVert.pt;
				else										p2	= ( thisVert.pt + nextVert.pt ) * 0.5;

				// polygonize the spline sub section.
				this->PolygonizeSplineSubSection( p0, thisVert.pt, p2, splineRecusLevel, interpolate, outPolygon );
			}
			break;
		case eVertexType_Arc:
		case eVertexType_Radius:
			{
				VWPoint2D		lastPt, nextPt;

				if ( lastVert.type == eVertexType_Corner )	lastPt	= lastVert.pt;
				else										lastPt	= ( lastVert.pt + thisVert.pt ) * 0.5;

				outPolygon.AddVertex( lastPt );				// add.

				if ( nextVert.type == eVertexType_Corner)	nextPt = nextVert.pt;
				else										nextPt = ( nextVert.pt + thisVert.pt ) * 0.5;

				if ( thisVert.type == eVertexType_Arc )
				{
					// polygonize the arc.
					this->PolygonizeArc( lastPt, thisVert.pt, nextPt, outPolygon );
				}
				else if ( thisVert.type == eVertexType_Radius )
				{
					// 'thisVert.pt' lies on the arc.
					VWArc2D			arc( lastPt, thisVert.pt, nextPt );	// form the arc.

					VWPolygon2D		arcPolygon;
					arc.GetPolygon( arcPolygon, arcDegRes );			// polygonize.

					outPolygon.AddVertex( arcPolygon );					// add resulting polygon.
				}
			}
			break;
	}
}

void VWPolygon2D::PolygonizeSplineSubSection(const VWPoint2D& v1, const VWPoint2D& v2, const VWPoint2D& v3, short level, bool interpolate, VWPolygon2D& outPolygon) const
{
	VWPoint2D		v12, v22, v23;
	VWPoint2D		v1A, v2A, v3A, v1B, v2B, v3B;

	VWLine2D		line;

	if ( level > 1 )					// now if it is called with level > 1, it processes
	{
		v12 = ( v1 + v2 )	* 0.5;		// 2 levels, to reduce the number of recursive
		v23 = ( v2 + v3 )	* 0.5;		// function calls, which makes it faster; also
		v22 = ( v12 + v23 ) * 0.5;		// it won't make a recursive call with level = 0,
		-- level;						// but will do the TDFLineTo [BF 11/7/02]

		v1A = v1;
		v2A = v12;
		v3A = v22;
		v1B = v22;
		v2B = v23;
		v3B = v3;

		v12 = ( v1A + v2A ) * 0.5;
		v23 = ( v2A + v3A ) * 0.5;
		v22 = ( v12 + v23 ) * 0.5;
		-- level;

		if ( level == 0 )
		{
			if ( interpolate )
			{
				outPolygon.AddVertex( v22, eVertexType_Cubic );
				outPolygon.AddVertex( v3A, eVertexType_Cubic );
			}
			else
			{
				outPolygon.AddVertex( v22 );
				outPolygon.AddVertex( v3A );
			}
		}
		else
		{
			// recursion.
			this->PolygonizeSplineSubSection( v1A, v12, v22, level, interpolate, outPolygon );
			this->PolygonizeSplineSubSection( v22, v23, v3A, level, interpolate, outPolygon );
		}

		v12 = ( v1B + v2B ) * 0.5;
		v23 = ( v2B + v3B ) * 0.5;
		v22 = ( v12 + v23 ) * 0.5;

		if ( level == 0 )
		{
			if ( interpolate )
			{
				outPolygon.AddVertex( v22, eVertexType_Cubic );
				outPolygon.AddVertex( v3B, eVertexType_Cubic );
			}
			else
			{
				outPolygon.AddVertex( v22 );
				outPolygon.AddVertex( v3B );
			}
		}
		else
		{
			// recursion.
			this->PolygonizeSplineSubSection( v1B, v12, v22, level, interpolate, outPolygon );
			this->PolygonizeSplineSubSection( v22, v23, v3B, level, interpolate, outPolygon );
		}
	}
	else if ( level == 1 )
	{
		v12 = ( v1 + v2 )	* 0.5;
		v23 = ( v2 + v3 )	* 0.5;
		v22 = ( v12 + v23 ) * 0.5;

		if ( interpolate )
		{
			outPolygon.AddVertex( v22, eVertexType_Cubic );
			outPolygon.AddVertex( v3, eVertexType_Cubic );
		}
		else
		{
			outPolygon.AddVertex( v22 );
			outPolygon.AddVertex( v3 );
		}
	}
	else
	{
		if ( interpolate )
		{
			outPolygon.AddVertex( v3, eVertexType_Cubic );
		}
		else
		{
			outPolygon.AddVertex( v3 );
		}
	}
}

bool VWPolygon2D::CalcChamfer(const VWPoint2D& pt1, const VWPoint2D& int0Value, const VWPoint2D& pt4, VWPoint2D& lastCam, VWPoint2D& thisCam, VWPoint2D& center, double& radius, double& lastAngle, double& thisAngle) const
{
	VWPoint2D	int0	= int0Value;
	double		shortLen;
	VWPoint2D	thisPerp, lastPerp, lastVec, thisVec;
	double		lastLen, thisLen;
	bool 		intersect, parallel;

	// find intersection.
	VWLine2D::IntersectLines( pt1, int0, int0, pt4, intersect, parallel, int0, kPolygonize_IntersEpsilon );

	if ( parallel )
	{
		return	false;
	}
	else
	{
		thisLen			= int0.DistanceTo( pt4 );
		lastLen			= int0.DistanceTo( pt1 );
		if ( thisLen != 0 )		thisVec = ( pt4 - int0 ) / thisLen;	// normalize.
		if ( lastLen != 0 )		lastVec = ( pt1 - int0 ) / lastLen;	// normalize.

		// if the radius is preset by user and the segments are long enough then use the preset radius.
		// if the lengths are within a certain percentage of eachother, don't bother with the Cams.
		if ( ( thisLen != 0 ) && ( Abs( ( lastLen / thisLen ) - 1 ) < 0.001 ) )
		{
			lastCam		= pt1;
			thisCam		= pt4;
		}
		else if ( lastLen < thisLen )
		{
			lastCam		= pt1;
			thisCam		= int0 + thisVec * lastLen;
		}
		else
		{
			thisCam		= pt4;
			lastCam		= int0 + lastVec * thisLen;
		}

		// arc part.
		thisPerp		= thisCam;
		VWPoint2D	perpVec( thisVec.y, -thisVec.x );
		thisPerp		-=	perpVec;
		lastPerp		= lastCam;
		perpVec.SetPoint( lastVec.y, -lastVec.x );
		lastPerp		-= perpVec;
		VWLine2D::IntersectLines( lastCam, lastPerp, thisCam, thisPerp, intersect, parallel, center, kPolygonize_IntersEpsilon );

		if ( parallel )
		{
			return false;
		}

		shortLen		= Min( ( center - thisCam ).Magnitude(), ( center - lastCam ).Magnitude() );

		radius			= shortLen;

		if ( parallel )
		{
			return false;
		}
		else
		{
			lastAngle	= MathUtils::FixedAngle( center, lastCam );
			thisAngle	= MathUtils::FixedAngle( center, thisCam );

			thisAngle	-= lastAngle;

			if ( thisAngle == 0 || abs( thisAngle ) == 360 )
			{
				return false;
			}
			else
			{
				lastAngle			= 90.0 - lastAngle;
				thisAngle			= -thisAngle;

				if ( thisAngle > 180.0 )
				{
					thisAngle		-= 360.0;
				}
				else if (-thisAngle > 180.0 )
				{
					thisAngle		+= 360.0;
				}

				if ( lastAngle < 0 )
				{
					do
					{
						lastAngle	+= 360.0;
					}
					while ( lastAngle <= 0 );
				}

				if ( lastAngle > 360.0 )
				{
					do
					{
						lastAngle	-= 360.0;
					}
					while ( lastAngle >= 360.0 );
				}
			}
		}

		return true;
	}
}

VWPoint2D VWPolygon2D::NextLoc(double a, const VWPoint2D& cen, double radius) const
{
	VWPoint2D	orientation(	1, 0 );
	VWPoint2D	perpDir(		0, 1 );

	return		( cen + radius * MathUtils::SpecialSin( a ) * orientation + radius * MathUtils::SpecialCos( a ) * perpDir );
}

void VWPolygon2D::PolygonizeArc(const VWPoint2D& lastPt, const VWPoint2D& thisPt, const VWPoint2D& nextPt, VWPolygon2D& outPolygon) const
{
	VWPoint2D	lastCam;
	VWPoint2D	thisCam;
	VWPoint2D	center;
	double		theStart;
	double		theArc;
	double		radius;

	// prepare the arc.
	if ( this->CalcChamfer( lastPt, thisPt, nextPt, lastCam, thisCam, center, radius, theStart, theArc ) )
	{
		this->PolygonizeArc( lastPt, nextPt, radius, lastCam, thisCam, center, theStart, theArc, outPolygon );
	}
}

void VWPolygon2D::PolygonizeArc(const VWPoint2D &lastPt, const VWPoint2D& nextPt, double radius, VWPoint2D& lastCam, VWPoint2D& thisCam, VWPoint2D& center, double theStart, double theArc, VWPolygon2D& outPolygon) const
{
	if ( lastPt != lastCam )
	{
		outPolygon.AddVertex( lastCam );
	}

	if ( theArc )
	{
		this->PolygonizeArcSegment( theStart, theArc, center, radius, outPolygon );
	}

	if ( nextPt != thisCam )
	{
		outPolygon.AddVertex( thisCam );
	}
}

void VWPolygon2D::PolygonizeArcSegment(double inStart, double inArc, const VWPoint2D& cen, double radius, VWPolygon2D& outPolygon) const
{
	if ( inArc < 0 )
	{
		inStart			+= 360.0;
	}

	double	theEnd		= inArc + inStart;

	if ( ( theEnd >= 360.0 ) && ( inStart > 360.0 ) )
	{
		theEnd 			-= 360.0;
		inStart			-= 360.0;
	}

	// start recursion level.
	short	startLevel	= 8;

	this->PolygonizeArcSegment( inStart, theEnd, startLevel, cen, radius, outPolygon );
}

void VWPolygon2D::PolygonizeArcSegment(double arcStart, double arcEnd, short currentLevel, const VWPoint2D& cen, double radius, VWPolygon2D& outPolygon) const
{
	double	midArc		= ( arcStart + arcEnd ) * 0.5;
	double	normalArc	= midArc;

	if ( midArc >= 360.0 )
	{
		normalArc		-= 360.0;
	}

	if ( currentLevel > 1 )
	{
		-- currentLevel;

		// recursion.
		this->PolygonizeArcSegment( arcStart, midArc, currentLevel, cen, radius, outPolygon );

		// add arc point.
		const VWPoint2D& nextPt	= this->NextLoc( normalArc, cen, radius );
		outPolygon.AddVertex( nextPt );

		// recursion.
		this->PolygonizeArcSegment( midArc, arcEnd, currentLevel, cen, radius, outPolygon );
	}
	else
	{
		const VWPoint2D&	nextPt		= this->NextLoc( midArc, cen, radius );
		outPolygon.AddVertex( nextPt );
	}
}