//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License
//


#include "StdHeaders.h"

//#include "STRResLoader.h"


using namespace VWFC::Tools;

// ---------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------
CSTRResLoader::CSTRResLoader()
{
}

// ---------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------
CSTRResLoader::~CSTRResLoader()
{
}

// ---------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------
/*static*/ CSTRResLoader& CSTRResLoader::Instance()
{
	static CSTRResLoader	instance;
	return instance;
}


// ---------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------
Uint16	CSTRResLoader::GetStrResourceCount( short id )
{
	Uint16 count = -1;

	GSHandle	hResource	= ::GS_GetResource( gCBP, 'STR#', id, true );
	if ( hResource && *hResource ) {
		count = **(Uint16**) hResource;
		::GS_ReleaseResource( gCBP, (GSHandle) hResource );
	}

	return count;
}

// ---------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------
bool CSTRResLoader::CacheSTR(short id, bool useOnlyLocalResources)
{
	bool result = true;
	TStringsMap::iterator	it	= fmapStrings.find( id );
	if ( it == fmapStrings.end() )
	{
		result = false;

		// add this string table
		GSHandle	hResource	= ::GS_GetResource( gCBP, 'STR#', id, useOnlyLocalResources );
		if ( hResource && *hResource )
		{
			Uint16		numStrings		= **(Uint16**) hResource;
			size_t		resourceSize;

			GS_GetHandleSize(gCBP, hResource, resourceSize);

			TStrResData &data = fmapStrings[id];
			data.fData.resize(resourceSize);
			data.fIndex.resize(numStrings+1);

			memcpy(&data.fData[0], *hResource, resourceSize);

			// skip items count and position on the first string
			const unsigned char *pStrings = &data.fData[2];
			const unsigned char *pResEnd = pStrings + resourceSize - 2;

			// read the strings
			if ( pStrings && numStrings )
			{
				for(Uint16 i=1; i<=numStrings && pStrings < pResEnd; i++ )
				{
					data.fIndex[i] = pStrings;
					pStrings += pStrings[ 0 ] + 1;
				}
			}
			
			// release resource
			::GS_ReleaseResource( gCBP, (GSHandle) hResource );

			result = true;
		}
	}

	return result;
}

// ---------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------
bool CSTRResLoader::Load(short id, short index, TXString& outString, bool useOnlyLocalResources)
{
	if (&outString != &fInternalStringForLoad) {
		TLoadedTXStringMap::iterator lsi = fLoadedStrings.find(TShortPair(id, index));
		if (lsi != fLoadedStrings.end()) {
			outString = lsi->second;
			return true;
		}
	}

	TStringsMap::iterator	it	= fmapStrings.find( id );
	if ( it == fmapStrings.end() ) {
		this->CacheSTR( id, useOnlyLocalResources );
	}

	// read the string
	bool	ok		= false;
	it				= fmapStrings.find( id );
	VERIFYN( kVStanev, it != fmapStrings.end() );
	if ( it != fmapStrings.end() ) {
		const TStrResData&	resData	= it->second;
		if ( index >= 1 && (size_t) index < resData.fIndex.size() ) {
			outString				= resData.fIndex[index];

			outString.AnsiToMac();

			typedef std::pair<TShortPair, TXString> MapPair;
			fLoadedStrings.insert(MapPair(TShortPair(id, index), outString));

			ok						= true;
		}
	}

	return ok;
}

// ---------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------
const TXString& CSTRResLoader::Load(short id, short index, bool useOnlyLocalResources)
{
	TLoadedTXStringMap::iterator lsi = fLoadedStrings.find(TShortPair(id, index));

	if (lsi != fLoadedStrings.end()) 
		return lsi->second;

	else if (!VERIFYN( kVStanev, this->Load( id, index, fInternalStringForLoad, useOnlyLocalResources) ))
		return fErrorReturnString;

	lsi = fLoadedStrings.find(TShortPair(id, index));
	if (lsi == fLoadedStrings.end()) 
		return fErrorReturnString;

	return fLoadedStrings[TShortPair(id, index)];
}

// ---------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------
bool CSTRResLoader::LoadText(short id, TXString& outString, bool useOnlyLocalResources /*= true*/)
{
	bool		result		= false;
	GSHandle	hResource	= ::GS_GetResource( gCBP, 'TEXT', id, useOnlyLocalResources );
	if ( hResource && *hResource ) {
		size_t	textSize	= 0;
		::GS_GetHandleSize( gCBP, hResource, textSize );
		if ( textSize > 0 ) {
			outString	= TXString( (const char*) *hResource, textSize );
		}

		result	= true;
	}

	return result;
}

// ---------------------------------------------------------------------------------------------------
// ---------------------------------------------------------------------------------------------------
TXString CSTRResLoader::LoadText(short id, bool useOnlyLocalResources /*= true*/)
{
	TXString	textStr;
	this->LoadText( id, textStr );
	return textStr;
}

