//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License
//


#include "StdHeaders.h"
//#include "VWString.h"

using namespace VWFC::Tools;


// --------------------------------------------------------------------------------------------------
/*static*/ void TXStringExt::Tokenize(const TXString& string, TXStringSTLArray& outArray, const char* szTokens, size_t tokensLen)
{
	outArray.clear();

	TXString	currToken;

	size_t	strLen	= string.GetLength();
	for(size_t i=0; i<strLen; i++) {
		char	ch		= string.GetAt( i );

		bool	isToken	= false;
		for(size_t j=0; j<tokensLen; j++) {
			if ( szTokens[j] == ch ) {
				isToken		= true;
				break;
			}
		}

		if ( isToken ) {
			outArray.push_back( currToken );
			currToken.Clear();
		}
		else {
			currToken		+= ch;
		}
	}

	// put the last item if any
	if ( ! currToken.IsEmpty() ) {
		outArray.push_back( currToken );
		currToken.Clear();
	}
}

/*static*/ bool TXStringExt::IsSpace(char c)
{
	return ( ((c) == ' ') || ((c) == '\t') || ((c) == '\n') || ((c) == '\r') );
}

/*static*/ bool TXStringExt::IsDigit(char c)
{
	return ( (((c) >= '0') && ((c) <= '9')) );
}

/*static*/ bool TXStringExt::IsAlpha(char c)
{
	return ( (((c) >= 'a') && ((c) <= 'z')) ) || ( (((c) >= 'A') && ((c) <= 'Z')) );
}

/*static*/ bool TXStringExt::IsDigitOrAlpha(char c)
{
	return TXStringExt::IsDigit( c ) || TXStringExt::IsAlpha( c ) || ((c) == '_');
}

/*static*/ char TXStringExt::ToLower(char c)
{
	return ( (((c) >= 'A') && ((c) <= 'Z')) ? ((c) - 'A' + 'a') : (c) );
}

/*static*/ char TXStringExt::ToUpper(char c)
{
	return ( (((c) >= 'a') && ((c) <= 'z')) ? ((c) - 'a' + 'A') : (c) );
}

/*static*/ char TXStringExt::ToReverseCase(char c)
{
	if ( ( (((c) >= 'a') && ((c) <= 'z')) ) )
		return TXStringExt::ToUpper( c );
	else
		return TXStringExt::ToLower( c );
}

/*static*/ bool TXStringExt::IsDigit(const TXString& string)
{
	size_t	i	= 0;
	if ( i >= string.GetLength() )
		return false;

	if ( string.GetAt(i) == '-' || string.GetAt(i) == '+' || string.GetAt(i) =='.' )
		i++;

	if ( i >= string.GetLength() )
		return false;

	return TXStringExt::IsDigit( string.GetAt(i) );
}

/*static*/ TXString TXStringExt::EncodeString(const TXString& string)
{
	TXString		result;
	TXString		encodedPiece;

	size_t	strLen	= string.GetLength();
	for(size_t i=0; i<strLen; i++) {
		char	ch		= string.GetAt( i );
		if ( TXStringExt::IsDigitOrAlpha( ch ) || ch == '_' )
		{
			result	+= ch;
		}
		else
		{
			encodedPiece.Format( "%X", ch );
			result	+= "%";
			result	+= encodedPiece;
		}
	}

	return result;
}

/*static*/ TXString TXStringExt::DecodeString(const TXString& string)
{
	TXString	result;

	size_t	strLen	= string.GetLength();
	for(size_t i=0; i<strLen; i++) {
		char	ch		= string.GetAt( i );
		if ( ch == '%' )
		{
			char	chResult	= 0;
			for(size_t j=0; j<2 && i<strLen; j++) {
				i++;
				ch					= string.GetAt( i );
				char	thisChar	= TXStringExt::IsDigit( ch ) ? (ch - L'0') : ((TXStringExt::ToUpper(ch) - L'A') + 10);
				chResult			= 16 * chResult + thisChar;
			}

			if ( chResult != 0 )
			{
				result += chResult;
			}
		}
		else
		{
			result	+= ch;
		}
	}

	return result;
}

/*static*/ TXString TXStringExt::Format(const char* format, ...) 
{ 
	TXString retStr;

	if ( NULL != format )
	{
		va_list marker = NULL; 

		// initialize variable arguments
		va_start(marker, format); 

		// Get formatted string length adding one for NULL
#if GS_WIN	
		size_t len = _vscprintf(format, marker) + 1;
#else
		size_t len = 10240;
#endif

		// Create a char vector to hold the formatted string.
		char*	 buffer	= new char[ len ];
#if GS_WIN	
		int nWritten = _vsnprintf(&buffer[0], len, format, marker);	// returns -1 if truncated
#else
		int nWritten = vsnprintf(&buffer[0], len, format, marker);	// returns -1 if truncated
#endif

		buffer[ nWritten ] = 0;

		if (nWritten > 0)
		{
			retStr = buffer;
		}

		delete [] buffer;
		buffer	= NULL;

		// Reset variable arguments
		va_end(marker);
	}

	return retStr; 
}

/*static*/ TXString TXStringExt::CapitalizeEachWord(const TXString& string)
{
	TXString result;
	bool isFirst = true;
	size_t	strLen	= string.GetLength();

	for(size_t i=0; i<strLen; i++) 
	{
		char	ch		= string.GetAt( i );
		if(TXStringExt::IsSpace(ch))
		{
			isFirst = true;
			result += ch;
		}
		else if(isFirst)
		{
			isFirst = false;
			result += TXStringExt::ToUpper(ch);
		}
		else
			result += TXStringExt::ToLower(ch);
	}

	return result;
}


//Utility to search from the end of string. 
/*static*/ int TXStringExt::FindLastSubString(const TXString& string, const TXString& sub)
{
	int found, maxFound = -1;
	found = string.Find(sub);
	while(found != -1)
	{
		maxFound = found;
		found = string.Find(found+sub.GetLength(), sub);
	}
	return maxFound;
}