//
//	Copyright  Nemetschek Vectorworks, Inc.
//	Use of this file is governed by the Nemetschek Vectorworks SDK License Agreement
//	http://developer.vectorworks.net/index.php?title=Vectorworks_SDK_License
//

#include "StdHeaders.h"

#include "ListBrowserItem.h"

using namespace VWFC::VWUI;
using namespace VWFC::Tools;

VWListBrowserItem::VWListBrowserItem(TControlID dialogID, TControlID controlID, short columnID, short rowID)
{
	fDialogID	= dialogID;
	fControlID	= controlID;
	fColumnID	= columnID;
	fRowID		= rowID;
}

VWListBrowserItem::VWListBrowserItem (const VWListBrowserItem& item)
{
	fDialogID	= item.fDialogID;
	fControlID	= item.fControlID;
	fColumnID	= item.fColumnID;
	fRowID		= item.fRowID;
}

VWListBrowserItem::~VWListBrowserItem()
{
}

VWListBrowserItem& VWListBrowserItem::operator=(const VWListBrowserItem& item)
{
	fDialogID	= item.fDialogID;
	fControlID	= item.fControlID;
	fColumnID	= item.fColumnID;
	fRowID		= item.fRowID;
	return *this;
}

void VWListBrowserItem::SetItemText(const TXString& text)
{
	VERIFYN( kVStanev, ::GS_SetListBrowserItemInfo( gCBP, fDialogID, fControlID, fRowID, fColumnID, text, -1 ) );
}

void VWListBrowserItem::SetItemImage(size_t imageIndex)
{
	TXString		text;
	VERIFYN( kVStanev, ::GS_SetListBrowserItemInfo( gCBP, fDialogID, fControlID, fRowID, fColumnID, text, DemoteTo<short> ( kVStanev, imageIndex ) ) );
}

void VWListBrowserItem::SetItemInfo(const TXString& text, size_t imageIndex)
{
	VERIFYN( kVStanev, ::GS_SetListBrowserItemInfo( gCBP, fDialogID, fControlID, fRowID, fColumnID, text, DemoteTo<short> ( kVStanev, imageIndex ) ) );
}

TXString VWListBrowserItem::GetItemText() const
{
	TXString	result;

	Str255		text;
	short		imageIndex = -1;
	if ( ::GS_GetListBrowserItemInfo( gCBP, fDialogID, fControlID, fRowID, fColumnID, text, imageIndex ) ) {
		result	= text;
	}

	return result;
}

size_t VWListBrowserItem::GetItemImage() const
{
	size_t		result		= (size_t) -1;

	Str255		text;
	short		imageIndex = -1;
	if ( ::GS_GetListBrowserItemInfo( gCBP, fDialogID, fControlID, fRowID, fColumnID, text, imageIndex ) ) {
		result	= imageIndex;
	}

	return result;
}

bool VWListBrowserItem::GetItemInfo(TXString& outText, size_t& outImage) const
{
	Str255 text;
	short  imageIndex = -1;
	if ( ::GS_GetListBrowserItemInfo( gCBP, fDialogID, fControlID, fRowID, fColumnID, text, imageIndex ) ) {
		outText			= text;
		outImage		= imageIndex;
		return true;
	}
	else {
		outText.Clear();
		outImage		= (size_t) -1;
		return false;
	}
}

bool VWListBrowserItem::SetItemData( const long userData )
{
	return GS_SetListBrowserItemData( gCBP, fDialogID, fControlID, fRowID, fColumnID, userData );
}

bool VWListBrowserItem::GetItemData( long &outUserData )
{
	return GS_GetListBrowserItemData( gCBP, fDialogID, fControlID, fRowID, fColumnID, outUserData );
}

void VWListBrowserItem::SetItemToolTip(const TXString& text)
{
	gSDK->SetItemToolTipText( fDialogID, fControlID, text, "", fRowID, fColumnID );
}

void VWListBrowserItem::SetItemToolTip(const TXString& text, const TXString& subText)
{
	gSDK->SetItemToolTipText( fDialogID, fControlID, text, subText, fRowID, fColumnID );
}

bool VWListBrowserItem::EnsureItemVisible() const
{
	return ::GS_EnsureListBrowserItemIsVisible( gCBP, fDialogID, fControlID, fRowID ) ? true : false;
}

void VWListBrowserItem::SetItemTextStyle(EItemTextStyle textStyle)
{
	VERIFYN( kVStanev, ::GS_SetListBrowserItemTextStyle( gCBP, fDialogID, fControlID, fRowID, fColumnID, DemoteTo<unsigned char> ( kVStanev, textStyle ) ) );
}

EItemTextStyle VWListBrowserItem::GetItemTextStyle()
{
	unsigned char textStyle;
	bool ok = ::GS_GetListBrowserItemTextStyle( gCBP, fDialogID, fControlID, fRowID, fColumnID, textStyle);
	return ok ? (EItemTextStyle) textStyle : (EItemTextStyle) -1;
}

void VWListBrowserItem::SetItemTextJustification(EItemTextJustification justification)
{
	VERIFYN( kVStanev, ::GS_SetListBrowserItemTextJust( gCBP, fDialogID, fControlID, fRowID, fColumnID, DemoteTo<int> ( kVStanev, justification ) ) );
}

EItemTextJustification VWListBrowserItem::GetItemTextJustification()
{
	int justification;
	bool ok = ::GS_GetListBrowserItemTextJust( gCBP, fDialogID, fControlID, fRowID, fColumnID, justification);
	return ok ? (EItemTextJustification) justification : (EItemTextJustification) -1;
}

void VWListBrowserItem::SetMultImageIndexes(size_t imageIndex0, size_t imageIndex1, size_t imageIndex2)
{
	VERIFYN( kVStanev, ::GS_SetListBrowserMultImageIndexes( gCBP, fDialogID, fControlID, fRowID, fColumnID, DemoteTo<int> ( kVStanev, imageIndex0 ), DemoteTo<int> ( kVStanev, imageIndex1 ), DemoteTo<int> ( kVStanev, imageIndex2 ) ) );
}

bool VWListBrowserItem::GetMultImageIndexes(size_t& outImageIndex0, size_t& outImageIndex1, size_t& outImageIndex2)
{
	int imageIndex0 = -1;
	int imageIndex1 = -1;
	int imageIndex2 = -1;

	if ( ::GS_GetListBrowserMultImageIndexes( gCBP, fDialogID, fControlID, fRowID, fColumnID, imageIndex0, imageIndex1, imageIndex2 ) ) {
		outImageIndex0		= imageIndex0;
		outImageIndex1		= imageIndex1;
		outImageIndex2		= imageIndex2;
		return true;
	}
	else {
		outImageIndex0		= (size_t) -1;
		outImageIndex1		= (size_t) -1;
		outImageIndex2		= (size_t) -1;
		return false;
	}
}

void VWListBrowserItem::SetOwnerDrawnType(EListBrowserOwnerDrawnType ownerDrawnType)
{
	VERIFYN( kVStanev, ::GS_SetListBrowserColumnOwnerDrawnType( gCBP, fDialogID, fControlID, fRowID, fColumnID, DemoteTo<int> ( kVStanev, ownerDrawnType ) ) );
}

EListBrowserOwnerDrawnType VWListBrowserItem::GetOwnerDrawnType()
{
	int ownerDrawnType;
	bool ok = ::GS_GetListBrowserColumnOwnerDrawnType( gCBP, fDialogID, fControlID, fRowID, fColumnID, ownerDrawnType );
	return ok ? (EListBrowserOwnerDrawnType) ownerDrawnType : kListBrowserNotOwnerDrawn;
}

void VWListBrowserItem::SetItemUsingColumnDataItem(size_t columnDataItemIndex)
{
	VERIFYN( kVStanev, ::GS_SetListBrowserItemUsingColumnDataItem( gCBP, fDialogID, fControlID, fRowID, fColumnID, DemoteTo<short> ( kVStanev, columnDataItemIndex ) ) );
}

bool VWListBrowserItem::GetColumnDataItemInfo()
{
	PStr255	text	= "";
	short imageOn	= -1;
	short imageOff	= -1;
	long  itemData	= -1;

	return ::GS_GetListBrowserColumnDataItemInfo( gCBP, fDialogID, fControlID, fColumnID, fRowID, text, imageOn, imageOff, itemData ) ? true : false;
}

// ----------------------------------------------------------------------------------------------------
// attributes
void VWListBrowserItem::SetItemTextColor(const CRGBColor& color)
{
	int		redIndex	= color.GetRed();
	int		greenIndex	= color.GetGreen();
	int		blueIndex	= color.GetBlue();

	VERIFYN( kVStanev, ::GS_SetListBrowserItemTextColor( gCBP, fDialogID, fControlID, fRowID, fColumnID, redIndex, greenIndex, blueIndex ) );
}

void VWListBrowserItem::SetItemTextColor(size_t redIndex, size_t greenIndex, size_t blueIndex)
{
	VERIFYN( kVStanev, ::GS_SetListBrowserItemTextColor( gCBP, fDialogID, fControlID, fRowID, fColumnID, DemoteTo<int> ( kVStanev, redIndex ), DemoteTo<int> ( kVStanev, greenIndex ), DemoteTo<int> ( kVStanev, blueIndex ) ) );
}

bool VWListBrowserItem::GetItemTextColor(CRGBColor& outColor)
{
	int redIndex	= -1;
	int greenIndex	= -1;
	int blueIndex	= -1;

	bool	ok	= ::GS_GetListBrowserItemTextColor( gCBP, fDialogID, fControlID, fRowID, fColumnID, redIndex, greenIndex, blueIndex);
	if ( ok )
	{
		outColor.SetColor( UInt8(redIndex), UInt8(greenIndex), UInt8(blueIndex) );
	}

	return ok;
}

void VWListBrowserItem::SetItemDashStyle(size_t styleIndex, size_t lineWeight)
{
	VERIFYN( kVStanev, ::GS_SetListBrowserItemDashStyle( gCBP, fDialogID, fControlID, fRowID, fColumnID, DemoteTo<int> ( kVStanev, styleIndex ), DemoteTo<int> ( kVStanev, lineWeight ) ) );
}

bool VWListBrowserItem::GetItemDashStyle()
{
	int styleIndex = -1;
	int lineWeight = -1;

	return ::GS_GetListBrowserItemDashStyle( gCBP, fDialogID, fControlID, fRowID, fColumnID, styleIndex, lineWeight ) ? true : false;
}

void VWListBrowserItem::SetItemFillForeColor(const CRGBColor& color)
{
	int		redIndex	= color.GetRed();
	int		greenIndex	= color.GetGreen();
	int		blueIndex	= color.GetBlue();

	VERIFYN( kVStanev, ::GS_SetListBrowserItemFillForeColor( gCBP, fDialogID, fControlID, fRowID, fColumnID, redIndex, greenIndex, blueIndex ) );
}

void VWListBrowserItem::SetItemFillForeColor(size_t redIndex, size_t greenIndex, size_t blueIndex)
{
	VERIFYN( kVStanev, ::GS_SetListBrowserItemFillForeColor( gCBP, fDialogID, fControlID, fRowID, fColumnID, DemoteTo<int> ( kVStanev, redIndex ), DemoteTo<int> ( kVStanev, greenIndex ), DemoteTo<int> ( kVStanev, blueIndex ) ) );
}

void VWListBrowserItem::SetItemNoFillForeColor()
{
	VERIFYN( kVStanev, ::GS_SetListBrowserItemFillForeColor( gCBP, fDialogID, fControlID, fRowID, fColumnID, -1, -1, -1 ) );
}

bool VWListBrowserItem::GetItemFillForeColor( size_t &outRedIndex, size_t &outGreenIndex, size_t &outBlueIndex )
{
	int		redIndex	= -1;
	int		greenIndex	= -1;
	int		blueIndex	= -1;
	bool	success = ::GS_GetListBrowserItemFillForeColor( gCBP, fDialogID, fControlID, fRowID, fColumnID, redIndex, greenIndex, blueIndex );
	
	outRedIndex = redIndex;
	outGreenIndex = greenIndex;
	outBlueIndex = blueIndex;
	
	return  success;
}

bool VWListBrowserItem::GetItemFillForeColor(CRGBColor& outColor)
{
	int		redIndex	= -1;
	int		greenIndex	= -1;
	int		blueIndex	= -1;
	bool	ok = ::GS_GetListBrowserItemFillForeColor( gCBP, fDialogID, fControlID, fRowID, fColumnID, redIndex, greenIndex, blueIndex );
	
	if ( ok )
	{
		outColor.SetColor( UInt8(redIndex), UInt8(greenIndex), UInt8(blueIndex) );
	}

	return ok;
}

void VWListBrowserItem::SetItemFillBackColor(const CRGBColor& color)
{
	int		redIndex	= color.GetRed();
	int		greenIndex	= color.GetGreen();
	int		blueIndex	= color.GetBlue();

	VERIFYN( kVStanev, ::GS_SetListBrowserItemFillBackColor( gCBP, fDialogID, fControlID, fRowID, fColumnID, redIndex, greenIndex, blueIndex ) );
}

void VWListBrowserItem::SetItemFillBackColor(size_t redIndex, size_t greenIndex, size_t blueIndex)
{
	VERIFYN( kVStanev, ::GS_SetListBrowserItemFillBackColor( gCBP, fDialogID, fControlID, fRowID, fColumnID, DemoteTo<int> ( kVStanev, redIndex ), DemoteTo<int> ( kVStanev, greenIndex ), DemoteTo<int> ( kVStanev, blueIndex ) ) );
}

bool VWListBrowserItem::GetItemFillBackColor(CRGBColor& outColor)
{
	int redIndex	= -1;
	int greenIndex	= -1;
	int blueIndex	= -1;

	bool ok = ::GS_GetListBrowserItemFillBackColor( gCBP, fDialogID, fControlID, fRowID, fColumnID, redIndex, greenIndex, blueIndex);
	if ( ok )
	{
		outColor.SetColor( UInt8(redIndex), UInt8(greenIndex), UInt8(blueIndex) );
	}

	return ok;
}

void VWListBrowserItem::SetItemGradientOrImageRefNumber(size_t refNumber)
{
	VERIFYN( kVStanev, ::GS_SetListBrowserItemGradientOrImageRefNumber( gCBP, fDialogID, fControlID, fRowID, fColumnID,  DemoteTo<long> ( kVStanev, refNumber ) ) );
}

size_t VWListBrowserItem::GetItemGradientOrImageRefNumber()
{
	long refNumber;
	bool ok = GS_GetListBrowserItemGradientOrImageRefNumber( gCBP, fDialogID, fControlID, fRowID, fColumnID, refNumber );
	return ok ? (size_t) refNumber : (size_t) -1;;
}

void VWListBrowserItem::SetItemTileRefNumber(size_t refNumber)
{
	VERIFYN(kVictor, gSDK->SetListBrowserItemTileRefNumber(fDialogID, fControlID, fRowID, fColumnID,  DemoteTo<long>(kVictor, refNumber)));
}

size_t VWListBrowserItem::GetItemTileRefNumber()
{
	long refNumber;
	bool ok = gSDK->GetListBrowserItemTileRefNumber(fDialogID, fControlID, fRowID, fColumnID, refNumber);
	return ok ? (size_t) refNumber : (size_t) -1;;
}

void VWListBrowserItem::SetItemPenForeColor(const CRGBColor& color)
{
	int		redIndex	= color.GetRed();
	int		greenIndex	= color.GetGreen();
	int		blueIndex	= color.GetBlue();

	VERIFYN( kVStanev, ::GS_SetListBrowserItemPenForeColor( gCBP, fDialogID, fControlID, fRowID, fColumnID, redIndex, greenIndex, blueIndex ) );
}

void VWListBrowserItem::SetItemPenForeColor(size_t redIndex, size_t greenIndex, size_t blueIndex)
{
	VERIFYN( kVStanev, ::GS_SetListBrowserItemPenForeColor( gCBP, fDialogID, fControlID, fRowID, fColumnID, DemoteTo<int> ( kVStanev, redIndex ), DemoteTo<int> ( kVStanev, greenIndex ), DemoteTo<int> ( kVStanev, blueIndex ) ) );
}

bool VWListBrowserItem::GetItemPenForeColor(CRGBColor& outColor)
{
	int redIndex	= -1;
	int greenIndex	= -1;
	int blueIndex	= -1;

	bool ok = ::GS_GetListBrowserItemPenForeColor( gCBP, fDialogID, fControlID, fRowID, fColumnID, redIndex, greenIndex, blueIndex);
	if ( ok )
	{
		outColor.SetColor( UInt8(redIndex), UInt8(greenIndex), UInt8(blueIndex) );
	}

	return ok;
}

void VWListBrowserItem::SetItemPenBackColor(const CRGBColor& color)
{
	int		redIndex	= color.GetRed();
	int		greenIndex	= color.GetGreen();
	int		blueIndex	= color.GetBlue();

	VERIFYN( kVStanev, ::GS_SetListBrowserItemPenBackColor( gCBP, fDialogID, fControlID, fRowID, fColumnID, redIndex, greenIndex, blueIndex ) );
}

void VWListBrowserItem::SetItemPenBackColor(size_t redIndex, size_t greenIndex, size_t blueIndex)
{
	VERIFYN( kVStanev, ::GS_SetListBrowserItemPenBackColor( gCBP, fDialogID, fControlID, fRowID, fColumnID, DemoteTo<int> ( kVStanev, redIndex ), DemoteTo<int> ( kVStanev, greenIndex ), DemoteTo<int> ( kVStanev, blueIndex ) ) );
}

bool VWListBrowserItem::GetItemPenBackColor(CRGBColor& outColor)
{
	int redIndex	= -1;
	int greenIndex	= -1;
	int blueIndex	= -1;

	bool ok = ::GS_GetListBrowserItemPenBackColor( gCBP, fDialogID, fControlID, fRowID, fColumnID, redIndex, greenIndex, blueIndex);
	if ( ok )
	{
		outColor.SetColor( UInt8(redIndex), UInt8(greenIndex), UInt8(blueIndex) );
	}

	return ok;
}







		